import hashlib
from datetime import datetime
from functools import wraps

from dateparser.data.languages_info import language_order
from .parser import date_order_chart
from .utils import registry


@registry
class Settings:
    """Control and configure default parsing behavior of dateparser.
    Currently, supported settings are:

    * `DATE_ORDER`
    * `PREFER_LOCALE_DATE_ORDER`
    * `TIMEZONE`
    * `TO_TIMEZONE`
    * `RETURN_AS_TIMEZONE_AWARE`
    * `PREFER_DAY_OF_MONTH`
    * `PREFER_DATES_FROM`
    * `RELATIVE_BASE`
    * `STRICT_PARSING`
    * `REQUIRE_PARTS`
    * `SKIP_TOKENS`
    * `NORMALIZE`
    * `RETURN_TIME_AS_PERIOD`
    * `PARSERS`
    * `DEFAULT_LANGUAGES`
    * `LANGUAGE_DETECTION_CONFIDENCE_THRESHOLD`
    * `CACHE_SIZE_LIMIT`
    """

    _default = True
    _pyfile_data = None
    _mod_settings = dict()

    def __init__(self, settings=None):
        if settings:
            self._updateall(settings.items())
        else:
            self._updateall(self._get_settings_from_pyfile().items())

    @classmethod
    def get_key(cls, settings=None):
        if not settings:
            return 'default'

        keys = sorted(['%s-%s' % (key, str(settings[key])) for key in settings])
        return hashlib.md5(''.join(keys).encode('utf-8')).hexdigest()

    @classmethod
    def _get_settings_from_pyfile(cls):
        if not cls._pyfile_data:
            from dateparser_data import settings
            cls._pyfile_data = settings.settings
        return cls._pyfile_data

    def _updateall(self, iterable):
        for key, value in iterable:
            setattr(self, key, value)

    def replace(self, mod_settings=None, **kwds):
        for k, v in kwds.items():
            if v is None:
                raise TypeError('Invalid {{"{}": {}}}'.format(k, v))

        for x in self._get_settings_from_pyfile().keys():
            kwds.setdefault(x, getattr(self, x))

        kwds['_default'] = False
        if mod_settings:
            kwds['_mod_settings'] = mod_settings

        return self.__class__(settings=kwds)


settings = Settings()


def apply_settings(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        mod_settings = kwargs.get('settings')
        kwargs['settings'] = mod_settings or settings

        if isinstance(kwargs['settings'], dict):
            kwargs['settings'] = settings.replace(mod_settings=mod_settings, **kwargs['settings'])

        if not isinstance(kwargs['settings'], Settings):
            raise TypeError("settings can only be either dict or instance of Settings class")

        return f(*args, **kwargs)
    return wrapper


class SettingValidationError(ValueError):
    pass


def _check_repeated_values(setting_name, setting_value):
    if len(setting_value) != len(set(setting_value)):
        raise SettingValidationError(
            'There are repeated values in the "{}" setting'.format(setting_name)
        )
    return


def _check_require_part(setting_name, setting_value):
    """Returns `True` if the provided list of parts contains valid values"""
    invalid_values = set(setting_value) - {'day', 'month', 'year'}
    if invalid_values:
        raise SettingValidationError(
            '"{}" setting contains invalid values: {}'.format(
                setting_name, ', '.join(invalid_values)
            )
        )
    _check_repeated_values(setting_name, setting_value)


def _check_parsers(setting_name, setting_value):
    """Returns `True` if the provided list of parsers contains valid values"""
    existing_parsers = [
        'timestamp', 'relative-time', 'custom-formats', 'absolute-time', 'no-spaces-time', 'negative-timestamp'
    ]  # FIXME: Extract the list of existing parsers from another place (#798)
    unknown_parsers = set(setting_value) - set(existing_parsers)
    if unknown_parsers:
        raise SettingValidationError(
            'Found unknown parsers in the "{}" setting: {}'.format(
                setting_name, ', '.join(unknown_parsers)
            )
        )
    _check_repeated_values(setting_name, setting_value)


def _check_default_languages(setting_name, setting_value):
    unsupported_languages = set(setting_value) - set(language_order)
    if unsupported_languages:
        raise SettingValidationError(
            "Found invalid languages in the '{}' setting: {}".format(
                setting_name, ', '.join(map(repr, unsupported_languages))
            )
        )
    _check_repeated_values(setting_name, setting_value)


def _check_between_0_and_1(setting_name, setting_value):
    is_valid = 0 <= setting_value <= 1
    if not is_valid:
        raise SettingValidationError(
            '{} is not a valid value for {}. It can take values between 0 and '
            '1.'.format(
                setting_value, setting_name,
            )
        )


def check_settings(settings):
    """
    Check if provided settings are valid, if not it raises `SettingValidationError`.
    Only checks for the modified settings.
    """
    settings_values = {
        'DATE_ORDER': {
            'values': tuple(date_order_chart.keys()),
            'type': str,
        },
        'TIMEZONE': {
            # we don't check invalid Timezones as they raise an error
            'type': str,
        },
        'TO_TIMEZONE': {
            # It defaults to None, but it's not allowed to use it directly
            # "values" can take unlimited options
            'type': str
        },
        'RETURN_AS_TIMEZONE_AWARE': {
            # It defaults to 'default', but it's not allowed to use it directly
            'type': bool
        },
        'PREFER_DAY_OF_MONTH': {
            'values': ('current', 'first', 'last'),
            'type': str
        },
        'PREFER_DATES_FROM': {
            'values': ('current_period', 'past', 'future'),
            'type': str,
        },
        'RELATIVE_BASE': {
            # "values" can take unlimited options
            'type': datetime
        },
        'STRICT_PARSING': {
            'type': bool
        },
        'REQUIRE_PARTS': {
            # "values" covered by the 'extra_check'
            'type': list,
            'extra_check': _check_require_part
        },
        'SKIP_TOKENS': {
            # "values" can take unlimited options
            'type': list,
        },
        'NORMALIZE': {
            'type': bool
        },
        'RETURN_TIME_AS_PERIOD': {
            'type': bool
        },
        'PARSERS': {
            # "values" covered by the 'extra_check'
            'type': list,
            'extra_check': _check_parsers
        },
        'FUZZY': {
            'type': bool
        },
        'PREFER_LOCALE_DATE_ORDER': {
            'type': bool
        },
        'DEFAULT_LANGUAGES': {
            'type': list,
            'extra_check': _check_default_languages
        },
        'LANGUAGE_DETECTION_CONFIDENCE_THRESHOLD': {
            'type': float,
            'extra_check': _check_between_0_and_1
        },
        'CACHE_SIZE_LIMIT': {
            'type': int,
        },
    }

    modified_settings = settings._mod_settings  # check only modified settings

    # check settings keys:
    for setting in modified_settings:
        if setting not in settings_values:
            raise SettingValidationError('"{}" is not a valid setting'.format(setting))

    for setting_name, setting_value in modified_settings.items():
        setting_type = type(setting_value)
        setting_props = settings_values[setting_name]

        # check type:
        if not setting_type == setting_props['type']:
            raise SettingValidationError(
                '"{}" must be "{}", not "{}".'.format(
                    setting_name, setting_props['type'].__name__, setting_type.__name__
                )
            )

        # check values:
        if setting_props.get('values') and setting_value not in setting_props['values']:
            raise SettingValidationError(
                '"{}" is not a valid value for "{}", it should be: "{}" or "{}"'.format(
                    setting_value,
                    setting_name,
                    '", "'.join(setting_props['values'][:-1]),
                    setting_props['values'][-1],
                )
            )

        # specific checks
        extra_check = setting_props.get('extra_check')
        if extra_check:
            extra_check(setting_name, setting_value)



info = {
    "name": "nmg",
    "date_order": "DMY",
    "january": [
        "ng1",
        "ngwɛn matáhra"
    ],
    "february": [
        "ng2",
        "ngwɛn ńmba"
    ],
    "march": [
        "ng3",
        "ngwɛn ńlal"
    ],
    "april": [
        "ng4",
        "ngwɛn ńna"
    ],
    "may": [
        "ng5",
        "ngwɛn ńtan"
    ],
    "june": [
        "ng6",
        "ngwɛn ńtuó"
    ],
    "july": [
        "ng7",
        "ngwɛn hɛmbuɛrí"
    ],
    "august": [
        "ng8",
        "ngwɛn lɔmbi"
    ],
    "september": [
        "ng9",
        "ngwɛn rɛbvuâ"
    ],
    "october": [
        "ng10",
        "ngwɛn wum"
    ],
    "november": [
        "ng11",
        "ngwɛn wum navǔr"
    ],
    "december": [
        "kris",
        "krísimin"
    ],
    "monday": [
        "mɔ́n",
        "mɔ́ndɔ"
    ],
    "tuesday": [
        "smb",
        "sɔ́ndɔ mafú mába"
    ],
    "wednesday": [
        "sml",
        "sɔ́ndɔ mafú málal"
    ],
    "thursday": [
        "smn",
        "sɔ́ndɔ mafú mána"
    ],
    "friday": [
        "mabágá má sukul",
        "mbs"
    ],
    "saturday": [
        "sas",
        "sásadi"
    ],
    "sunday": [
        "sɔ́n",
        "sɔ́ndɔ"
    ],
    "am": [
        "maná"
    ],
    "pm": [
        "kugú"
    ],
    "year": [
        "mbvu"
    ],
    "month": [
        "ngwɛn"
    ],
    "week": [
        "sɔ́ndɔ"
    ],
    "day": [
        "duö"
    ],
    "hour": [
        "wulā"
    ],
    "minute": [
        "mpálâ"
    ],
    "second": [
        "nyiɛl"
    ],
    "relative-type": {
        "0 day ago": [
            "dɔl"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "nakugú"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "namáná"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ro",
    "date_order": "DMY",
    "january": [
        "ian",
        "ianuarie"
    ],
    "february": [
        "feb",
        "februarie",
        "Febr"
    ],
    "march": [
        "mar",
        "martie",
        "Mart"
    ],
    "april": [
        "apr",
        "aprilie"
    ],
    "may": [
        "mai"
    ],
    "june": [
        "iun",
        "iunie"
    ],
    "july": [
        "iul",
        "iulie"
    ],
    "august": [
        "aug",
        "august"
    ],
    "september": [
        "sept",
        "septembrie",
        "Sep"
    ],
    "october": [
        "oct",
        "octombrie"
    ],
    "november": [
        "noiembrie",
        "nov",
        "Noiem"
    ],
    "december": [
        "dec",
        "decembrie"
    ],
    "monday": [
        "lun",
        "luni"
    ],
    "tuesday": [
        "mar",
        "marți"
    ],
    "wednesday": [
        "mie",
        "miercuri",
        "Mi"
    ],
    "thursday": [
        "joi"
    ],
    "friday": [
        "vin",
        "vineri"
    ],
    "saturday": [
        "sâm",
        "sâmbătă"
    ],
    "sunday": [
        "dum",
        "duminică"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "an",
        "ani"
    ],
    "month": [
        "lună",
        "luna",
        "luni"
    ],
    "week": [
        "săpt",
        "săptămână",
        "săptămâni"
    ],
    "day": [
        "zi",
        "zile"
    ],
    "hour": [
        "h",
        "oră",
        "ore"
    ],
    "minute": [
        "m",
        "min",
        "minut",
        "minute"
    ],
    "second": [
        "s",
        "sec",
        "secundă",
        "secunde"
    ],
    "relative-type": {
        "0 day ago": [
            "azi"
        ],
        "0 hour ago": [
            "ora aceasta"
        ],
        "0 minute ago": [
            "minutul acesta"
        ],
        "0 month ago": [
            "luna aceasta"
        ],
        "0 second ago": [
            "acum"
        ],
        "0 week ago": [
            "săptămâna aceasta"
        ],
        "0 year ago": [
            "anul acesta"
        ],
        "1 day ago": [
            "ieri"
        ],
        "1 month ago": [
            "luna trecută"
        ],
        "1 week ago": [
            "săptămâna trecută"
        ],
        "1 year ago": [
            "anul trecut"
        ],
        "in 1 day": [
            "mâine"
        ],
        "in 1 month": [
            "luna viitoare"
        ],
        "in 1 week": [
            "săptămâna viitoare"
        ],
        "in 1 year": [
            "anul viitor"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "acum (\\d+[.,]?\\d*) de zile",
            "acum (\\d+[.,]?\\d*) zi",
            "acum (\\d+[.,]?\\d*) zile"
        ],
        "\\1 hour ago": [
            "acum (\\d+[.,]?\\d*) de ore",
            "acum (\\d+[.,]?\\d*) h",
            "acum (\\d+[.,]?\\d*) oră"
        ],
        "\\1 minute ago": [
            "acum (\\d+[.,]?\\d*) de minute",
            "acum (\\d+[.,]?\\d*) min",
            "acum (\\d+[.,]?\\d*) minut"
        ],
        "\\1 month ago": [
            "acum (\\d+[.,]?\\d*) de luni",
            "acum (\\d+[.,]?\\d*) luni",
            "acum (\\d+[.,]?\\d*) lună"
        ],
        "\\1 second ago": [
            "acum (\\d+[.,]?\\d*) de secunde",
            "acum (\\d+[.,]?\\d*) sec",
            "acum (\\d+[.,]?\\d*) secundă"
        ],
        "\\1 week ago": [
            "acum (\\d+[.,]?\\d*) de săptămâni",
            "acum (\\d+[.,]?\\d*) săpt",
            "acum (\\d+[.,]?\\d*) săptămână"
        ],
        "\\1 year ago": [
            "acum (\\d+[.,]?\\d*) an",
            "acum (\\d+[.,]?\\d*) de ani"
        ],
        "in \\1 day": [
            "peste (\\d+[.,]?\\d*) de zile",
            "peste (\\d+[.,]?\\d*) zi",
            "peste (\\d+[.,]?\\d*) zile"
        ],
        "in \\1 hour": [
            "peste (\\d+[.,]?\\d*) de ore",
            "peste (\\d+[.,]?\\d*) h",
            "peste (\\d+[.,]?\\d*) oră"
        ],
        "in \\1 minute": [
            "peste (\\d+[.,]?\\d*) de minute",
            "peste (\\d+[.,]?\\d*) min",
            "peste (\\d+[.,]?\\d*) minut"
        ],
        "in \\1 month": [
            "peste (\\d+[.,]?\\d*) de luni",
            "peste (\\d+[.,]?\\d*) luni",
            "peste (\\d+[.,]?\\d*) lună"
        ],
        "in \\1 second": [
            "peste (\\d+[.,]?\\d*) de secunde",
            "peste (\\d+[.,]?\\d*) sec",
            "peste (\\d+[.,]?\\d*) secundă"
        ],
        "in \\1 week": [
            "peste (\\d+[.,]?\\d*) de săptămâni",
            "peste (\\d+[.,]?\\d*) săpt",
            "peste (\\d+[.,]?\\d*) săptămână"
        ],
        "in \\1 year": [
            "peste (\\d+[.,]?\\d*) an",
            "peste (\\d+[.,]?\\d*) ani",
            "peste (\\d+[.,]?\\d*) de ani"
        ]
    },
    "locale_specific": {
        "ro-MD": {
            "name": "ro-MD"
        }
    },
    "skip": [
        "de",
        "la",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "sentence_splitter_group": 1,
    "ago": [
        "în urmă"
    ],
    "in": [
        "în"
    ]
}



info = {
    "name": "ks",
    "date_order": "MDY",
    "january": [
        "جنؤری"
    ],
    "february": [
        "فرؤری"
    ],
    "march": [
        "مارٕچ"
    ],
    "april": [
        "اپریل"
    ],
    "may": [
        "میٔ"
    ],
    "june": [
        "جوٗن"
    ],
    "july": [
        "جوٗلایی"
    ],
    "august": [
        "اگست"
    ],
    "september": [
        "ستمبر"
    ],
    "october": [
        "اکتوٗبر"
    ],
    "november": [
        "نومبر"
    ],
    "december": [
        "دسمبر"
    ],
    "monday": [
        "ژٔنٛدرٕروار",
        "ژٔنٛدٕروار"
    ],
    "tuesday": [
        "بوٚموار"
    ],
    "wednesday": [
        "بودوار"
    ],
    "thursday": [
        "برٛٮ۪سوار"
    ],
    "friday": [
        "جُمہ"
    ],
    "saturday": [
        "بٹوار"
    ],
    "sunday": [
        "آتھوار",
        "اَتھوار"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "ؤری"
    ],
    "month": [
        "رٮ۪تھ"
    ],
    "week": [
        "ہفتہٕ"
    ],
    "day": [
        "دۄہ"
    ],
    "hour": [
        "گٲنٛٹہٕ"
    ],
    "minute": [
        "مِنَٹ"
    ],
    "second": [
        "سٮ۪کَنڑ"
    ],
    "relative-type": {
        "0 day ago": [
            "اَز"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "راتھ"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "پگاہ"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "qu",
    "date_order": "DMY",
    "january": [
        "qul",
        "qulla puquy"
    ],
    "february": [
        "hat",
        "hatun puquy"
    ],
    "march": [
        "pau",
        "pauqar waray"
    ],
    "april": [
        "ayr",
        "ayriwa"
    ],
    "may": [
        "aym",
        "aymuray"
    ],
    "june": [
        "int",
        "inti raymi"
    ],
    "july": [
        "ant",
        "anta sitwa"
    ],
    "august": [
        "qha",
        "qhapaq sitwa"
    ],
    "september": [
        "uma",
        "uma raymi"
    ],
    "october": [
        "kan",
        "kantaray"
    ],
    "november": [
        "aya",
        "ayamarq'a"
    ],
    "december": [
        "kap",
        "kapaq raymi"
    ],
    "monday": [
        "lun",
        "lunes"
    ],
    "tuesday": [
        "mar",
        "martes"
    ],
    "wednesday": [
        "mié",
        "miércoles"
    ],
    "thursday": [
        "jue",
        "jueves"
    ],
    "friday": [
        "vie",
        "viernes"
    ],
    "saturday": [
        "sab",
        "sábado"
    ],
    "sunday": [
        "dom",
        "domingo"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "year"
    ],
    "month": [
        "month"
    ],
    "week": [
        "week"
    ],
    "day": [
        "day"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "today"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {
        "qu-BO": {
            "name": "qu-BO"
        },
        "qu-EC": {
            "name": "qu-EC"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ce",
    "date_order": "YMD",
    "january": [
        "янв",
        "январь"
    ],
    "february": [
        "фев",
        "февраль"
    ],
    "march": [
        "мар",
        "март"
    ],
    "april": [
        "апр",
        "апрель"
    ],
    "may": [
        "май"
    ],
    "june": [
        "июн",
        "июнь"
    ],
    "july": [
        "июл",
        "июль"
    ],
    "august": [
        "авг",
        "август"
    ],
    "september": [
        "сен",
        "сентябрь"
    ],
    "october": [
        "окт",
        "октябрь"
    ],
    "november": [
        "ноя",
        "ноябрь"
    ],
    "december": [
        "дек",
        "декабрь"
    ],
    "monday": [
        "оршотан де"
    ],
    "tuesday": [
        "шинарин де"
    ],
    "wednesday": [
        "кхаарин де"
    ],
    "thursday": [
        "еарин де"
    ],
    "friday": [
        "пӏераскан де"
    ],
    "saturday": [
        "шот де"
    ],
    "sunday": [
        "кӏиранан де"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "ш",
        "шо"
    ],
    "month": [
        "бут",
        "бутт"
    ],
    "week": [
        "кӏир",
        "кӏира"
    ],
    "day": [
        "де"
    ],
    "hour": [
        "сахь",
        "сахьт"
    ],
    "minute": [
        "мин",
        "минот"
    ],
    "second": [
        "сек",
        "секунд"
    ],
    "relative-type": {
        "0 day ago": [
            "тахана"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "карарчу баттахь"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "карарчу кӏирнахь"
        ],
        "0 year ago": [
            "карарчу шарахь"
        ],
        "1 day ago": [
            "селхана"
        ],
        "1 month ago": [
            "баханчу баттахь"
        ],
        "1 week ago": [
            "даханчу кӏирнахь"
        ],
        "1 year ago": [
            "даханчу шарахь"
        ],
        "in 1 day": [
            "кхана"
        ],
        "in 1 month": [
            "рогӏерчу баттахь"
        ],
        "in 1 week": [
            "рогӏерчу кӏирнахь"
        ],
        "in 1 year": [
            "рогӏерчу шарахь"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) д хьалха",
            "(\\d+[.,]?\\d*) де хьалха"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) сахь хьалха",
            "(\\d+[.,]?\\d*) сахьт хьалха"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) мин хьалха",
            "(\\d+[.,]?\\d*) минот хьалха"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) б хьалха",
            "(\\d+[.,]?\\d*) бутт хьалха"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) сек хьалха",
            "(\\d+[.,]?\\d*) секунд хьалха"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) кӏир хьалха",
            "(\\d+[.,]?\\d*) кӏира хьалха"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) ш хьалха",
            "(\\d+[.,]?\\d*) шо хьалха"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) д даьлча",
            "(\\d+[.,]?\\d*) де даьлча"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) сахь даьлча",
            "(\\d+[.,]?\\d*) сахьт даьлча"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) мин яьлча",
            "(\\d+[.,]?\\d*) минот яьлча"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) б баьлча",
            "(\\d+[.,]?\\d*) бутт баьлча"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) сек яьлча",
            "(\\d+[.,]?\\d*) секунд яьлча"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) кӏир даьлча",
            "(\\d+[.,]?\\d*) кӏира даьлча"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) ш даьлча",
            "(\\d+[.,]?\\d*) шо даьлча"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ewo",
    "date_order": "DMY",
    "january": [
        "ngo",
        "ngɔn osú"
    ],
    "february": [
        "ngb",
        "ngɔn bɛ̌"
    ],
    "march": [
        "ngl",
        "ngɔn lála"
    ],
    "april": [
        "ngn",
        "ngɔn nyina"
    ],
    "may": [
        "ngt",
        "ngɔn tána"
    ],
    "june": [
        "ngs",
        "ngɔn saməna"
    ],
    "july": [
        "ngz",
        "ngɔn zamgbála"
    ],
    "august": [
        "ngm",
        "ngɔn mwom"
    ],
    "september": [
        "nge",
        "ngɔn ebulú"
    ],
    "october": [
        "nga",
        "ngɔn awóm"
    ],
    "november": [
        "ngad",
        "ngɔn awóm ai dziá"
    ],
    "december": [
        "ngab",
        "ngɔn awóm ai bɛ̌"
    ],
    "monday": [
        "mɔ́n",
        "mɔ́ndi"
    ],
    "tuesday": [
        "smb",
        "sɔ́ndɔ məlú mə́bɛ̌"
    ],
    "wednesday": [
        "sml",
        "sɔ́ndɔ məlú mə́lɛ́"
    ],
    "thursday": [
        "smn",
        "sɔ́ndɔ məlú mə́nyi"
    ],
    "friday": [
        "fúl",
        "fúladé"
    ],
    "saturday": [
        "sér",
        "séradé"
    ],
    "sunday": [
        "sɔ́n",
        "sɔ́ndɔ"
    ],
    "am": [
        "kíkíríg"
    ],
    "pm": [
        "ngəgógəle"
    ],
    "year": [
        "m̀bú"
    ],
    "month": [
        "ngɔn"
    ],
    "week": [
        "sɔ́ndɔ"
    ],
    "day": [
        "amǒs"
    ],
    "hour": [
        "awola"
    ],
    "minute": [
        "enútɛn"
    ],
    "second": [
        "akábəga"
    ],
    "relative-type": {
        "0 day ago": [
            "aná"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "angogé"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "okírí"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "sk",
    "date_order": "DMY",
    "january": [
        "jan",
        "január",
        "januára"
    ],
    "february": [
        "feb",
        "február",
        "februára"
    ],
    "march": [
        "mar",
        "marca",
        "marec"
    ],
    "april": [
        "apr",
        "apríl",
        "apríla"
    ],
    "may": [
        "máj",
        "mája"
    ],
    "june": [
        "jún",
        "júna"
    ],
    "july": [
        "júl",
        "júla"
    ],
    "august": [
        "aug",
        "august",
        "augusta"
    ],
    "september": [
        "sep",
        "september",
        "septembra"
    ],
    "october": [
        "okt",
        "október",
        "októbra"
    ],
    "november": [
        "nov",
        "november",
        "novembra"
    ],
    "december": [
        "dec",
        "december",
        "decembra"
    ],
    "monday": [
        "po",
        "pondelok"
    ],
    "tuesday": [
        "ut",
        "utorok"
    ],
    "wednesday": [
        "st",
        "streda",
        "stredu"
    ],
    "thursday": [
        "št",
        "štvrtok"
    ],
    "friday": [
        "pi",
        "piatok"
    ],
    "saturday": [
        "so",
        "sobota",
        "sobotu"
    ],
    "sunday": [
        "ne",
        "nedeľa",
        "nedeľu"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "r",
        "rok"
    ],
    "month": [
        "mes",
        "mesiac"
    ],
    "week": [
        "týž",
        "týždeň"
    ],
    "day": [
        "d",
        "deň"
    ],
    "hour": [
        "h",
        "hodina"
    ],
    "minute": [
        "min",
        "minúta"
    ],
    "second": [
        "s",
        "sekunda"
    ],
    "relative-type": {
        "0 day ago": [
            "dnes"
        ],
        "0 hour ago": [
            "v tejto hodine"
        ],
        "0 minute ago": [
            "v tejto minúte"
        ],
        "0 month ago": [
            "tento mesiac"
        ],
        "0 second ago": [
            "teraz"
        ],
        "0 week ago": [
            "tento týždeň"
        ],
        "0 year ago": [
            "tento rok"
        ],
        "1 day ago": [
            "včera"
        ],
        "1 month ago": [
            "minulý mesiac"
        ],
        "1 week ago": [
            "minulý týždeň"
        ],
        "1 year ago": [
            "minulý rok"
        ],
        "in 1 day": [
            "zajtra"
        ],
        "in 1 month": [
            "budúci mesiac"
        ],
        "in 1 week": [
            "budúci týždeň"
        ],
        "in 1 year": [
            "budúci rok"
        ],
        "2 day ago": [
            "predvčerom"
        ],
        "in 2 days": [
            "pozajtra"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "pred (\\d+[.,]?\\d*) d",
            "pred (\\d+[.,]?\\d*) dňami",
            "pred (\\d+[.,]?\\d*) dňom"
        ],
        "\\1 hour ago": [
            "pred (\\d+[.,]?\\d*) h",
            "pred (\\d+[.,]?\\d*) hodinami",
            "pred (\\d+[.,]?\\d*) hodinou"
        ],
        "\\1 minute ago": [
            "pred (\\d+[.,]?\\d*) min",
            "pred (\\d+[.,]?\\d*) minútami",
            "pred (\\d+[.,]?\\d*) minútou"
        ],
        "\\1 month ago": [
            "pred (\\d+[.,]?\\d*) mes",
            "pred (\\d+[.,]?\\d*) mesiacmi",
            "pred (\\d+[.,]?\\d*) mesiacom"
        ],
        "\\1 second ago": [
            "pred (\\d+[.,]?\\d*) s",
            "pred (\\d+[.,]?\\d*) sekundami",
            "pred (\\d+[.,]?\\d*) sekundou"
        ],
        "\\1 week ago": [
            "pred (\\d+[.,]?\\d*) týž",
            "pred (\\d+[.,]?\\d*) týždňami",
            "pred (\\d+[.,]?\\d*) týždňom"
        ],
        "\\1 year ago": [
            "pred (\\d+[.,]?\\d*) r",
            "pred (\\d+[.,]?\\d*) rokmi",
            "pred (\\d+[.,]?\\d*) rokom"
        ],
        "in \\1 day": [
            "o (\\d+[.,]?\\d*) d",
            "o (\\d+[.,]?\\d*) deň",
            "o (\\d+[.,]?\\d*) dní",
            "o (\\d+[.,]?\\d*) dni"
        ],
        "in \\1 hour": [
            "o (\\d+[.,]?\\d*) h",
            "o (\\d+[.,]?\\d*) hodinu",
            "o (\\d+[.,]?\\d*) hodín",
            "o (\\d+[.,]?\\d*) hodiny"
        ],
        "in \\1 minute": [
            "o (\\d+[.,]?\\d*) min",
            "o (\\d+[.,]?\\d*) minút",
            "o (\\d+[.,]?\\d*) minútu",
            "o (\\d+[.,]?\\d*) minúty"
        ],
        "in \\1 month": [
            "o (\\d+[.,]?\\d*) mes",
            "o (\\d+[.,]?\\d*) mesiac",
            "o (\\d+[.,]?\\d*) mesiacov",
            "o (\\d+[.,]?\\d*) mesiace"
        ],
        "in \\1 second": [
            "o (\\d+[.,]?\\d*) s",
            "o (\\d+[.,]?\\d*) sekundu",
            "o (\\d+[.,]?\\d*) sekúnd",
            "o (\\d+[.,]?\\d*) sekundy"
        ],
        "in \\1 week": [
            "o (\\d+[.,]?\\d*) týž",
            "o (\\d+[.,]?\\d*) týždeň",
            "o (\\d+[.,]?\\d*) týždňov",
            "o (\\d+[.,]?\\d*) týždne"
        ],
        "in \\1 year": [
            "o (\\d+[.,]?\\d*) r",
            "o (\\d+[.,]?\\d*) rok",
            "o (\\d+[.,]?\\d*) rokov",
            "o (\\d+[.,]?\\d*) roky"
        ]
    },
    "locale_specific": {},
    "skip": [
        "v",
        "vo",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "sentence_splitter_group": 1,
    "ago": [
        "pred"
    ],
    "in": [
        "o",
        "za"
    ],
    "simplifications": [
        {
            "pred sekundou": "pred 1 sekundou"
        },
        {
            "pred minútou": "pred 1 minútou"
        },
        {
            "pred hodinou": "pred 1 hodinou"
        },
        {
            "pred týždňom": "pred 1 týždňom"
        },
        {
            "pred mesiacom": "pred 1 mesiacom"
        },
        {
            "pred rokom": "pred 1 rokom"
        },
        {
            "pred pol rokom": "pred 6 mesiacmi"
        },
        {
            "o sekundu": "o 1 sekundu"
        },
        {
            "o minútu": "o 1 minútu"
        },
        {
            "o hodinu": "o 1 hodinu"
        },
        {
            "o týždeň": "o 1 týždeň"
        },
        {
            "o mesiac": "o 1 mesiac"
        },
        {
            "o rok": "o 1 rok"
        },
        {
            "o pol roka": "o 6 mesiacov"
        }
    ]
}



import argparse
import logging

from .fasttext_manager import fasttext_downloader
from .utils import clear_cache


def entrance():
    dateparser_argparse = argparse.ArgumentParser(
        description='dateparser download manager.'
    )
    dateparser_argparse.add_argument(
        '--fasttext',
        type=str,
        help='To download a fasttext language detection models. Supported models are "small" and "large"'
    )
    dateparser_argparse.add_argument(
        '--clear',
        '--clear-cache',
        help='To clear all cached models',
        action='store_true'
    )

    args = dateparser_argparse.parse_args()

    if args.clear:
        clear_cache()
        logging.info("dateparser-download: All cache deleted")

    if args.fasttext:
        fasttext_downloader(args.fasttext)

    if not (args.clear or args.fasttext):
        dateparser_argparse.error("dateparser-download: You need to specify the command (i.e.: --fasttext or --clear)")



info = {
    "name": "smn",
    "date_order": "DMY",
    "january": [
        "uđiv",
        "uđđâivemáánu"
    ],
    "february": [
        "kuovâ",
        "kuovâmáánu"
    ],
    "march": [
        "njuhčâ",
        "njuhčâmáánu"
    ],
    "april": [
        "cuáŋui",
        "cuáŋuimáánu"
    ],
    "may": [
        "vyesi",
        "vyesimáánu"
    ],
    "june": [
        "kesi",
        "kesimáánu"
    ],
    "july": [
        "syeini",
        "syeinimáánu"
    ],
    "august": [
        "porge",
        "porgemáánu"
    ],
    "september": [
        "čohčâ",
        "čohčâmáánu"
    ],
    "october": [
        "roovvâd",
        "roovvâdmáánu"
    ],
    "november": [
        "skammâ",
        "skammâmáánu"
    ],
    "december": [
        "juovlâ",
        "juovlâmáánu"
    ],
    "monday": [
        "vuo",
        "vuossaargâ",
        "vuossargâ"
    ],
    "tuesday": [
        "maj",
        "majebaargâ",
        "majebargâ"
    ],
    "wednesday": [
        "kos",
        "koskoho",
        "koskokko"
    ],
    "thursday": [
        "tuo",
        "tuorâstuv",
        "tuorâstâh"
    ],
    "friday": [
        "vás",
        "vástuppeeivi",
        "vástuppeivi"
    ],
    "saturday": [
        "láv",
        "lávurduv",
        "lávurdâh"
    ],
    "sunday": [
        "pas",
        "pasepeeivi",
        "pasepeivi"
    ],
    "am": [
        "ip"
    ],
    "pm": [
        "ep"
    ],
    "year": [
        "year"
    ],
    "month": [
        "month"
    ],
    "week": [
        "week"
    ],
    "day": [
        "day"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "today"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "kab",
    "date_order": "DMY",
    "january": [
        "yen",
        "yennayer"
    ],
    "february": [
        "fur",
        "fuṛar"
    ],
    "march": [
        "meɣ",
        "meɣres"
    ],
    "april": [
        "yeb",
        "yebrir"
    ],
    "may": [
        "may",
        "mayyu"
    ],
    "june": [
        "yun",
        "yunyu"
    ],
    "july": [
        "yul",
        "yulyu"
    ],
    "august": [
        "ɣuc",
        "ɣuct"
    ],
    "september": [
        "cte",
        "ctembeṛ"
    ],
    "october": [
        "tub",
        "tubeṛ"
    ],
    "november": [
        "nun",
        "nunembeṛ"
    ],
    "december": [
        "duǧ",
        "duǧembeṛ"
    ],
    "monday": [
        "san",
        "sanass"
    ],
    "tuesday": [
        "kraḍ",
        "kraḍass"
    ],
    "wednesday": [
        "kuẓ",
        "kuẓass"
    ],
    "thursday": [
        "sam",
        "samass"
    ],
    "friday": [
        "sḍis",
        "sḍisass"
    ],
    "saturday": [
        "say",
        "sayass"
    ],
    "sunday": [
        "yan",
        "yanass"
    ],
    "am": [
        "n tufat"
    ],
    "pm": [
        "n tmeddit"
    ],
    "year": [
        "aseggas"
    ],
    "month": [
        "aggur"
    ],
    "week": [
        "ddurt"
    ],
    "day": [
        "ass"
    ],
    "hour": [
        "tamert"
    ],
    "minute": [
        "tamrect"
    ],
    "second": [
        "tasint"
    ],
    "relative-type": {
        "0 day ago": [
            "ass-a"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "iḍelli"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "azekka"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "mk",
    "date_order": "DMY",
    "january": [
        "јан",
        "јануари"
    ],
    "february": [
        "фев",
        "февруари"
    ],
    "march": [
        "мар",
        "март"
    ],
    "april": [
        "апр",
        "април"
    ],
    "may": [
        "мај"
    ],
    "june": [
        "јун",
        "јуни"
    ],
    "july": [
        "јул",
        "јули"
    ],
    "august": [
        "авг",
        "август"
    ],
    "september": [
        "септ",
        "септември"
    ],
    "october": [
        "окт",
        "октомври"
    ],
    "november": [
        "ноем",
        "ноември"
    ],
    "december": [
        "дек",
        "декември"
    ],
    "monday": [
        "пон",
        "понеделник"
    ],
    "tuesday": [
        "вт",
        "вто",
        "вторник"
    ],
    "wednesday": [
        "сре",
        "среда"
    ],
    "thursday": [
        "чет",
        "четврток"
    ],
    "friday": [
        "пет",
        "петок"
    ],
    "saturday": [
        "саб",
        "сабота"
    ],
    "sunday": [
        "нед",
        "недела"
    ],
    "am": [
        "претпл",
        "претпладне"
    ],
    "pm": [
        "попл",
        "попладне"
    ],
    "year": [
        "год",
        "година"
    ],
    "month": [
        "мес",
        "месец"
    ],
    "week": [
        "недела",
        "сед"
    ],
    "day": [
        "ден"
    ],
    "hour": [
        "час"
    ],
    "minute": [
        "мин",
        "минута"
    ],
    "second": [
        "сек",
        "секунда"
    ],
    "relative-type": {
        "0 day ago": [
            "денес"
        ],
        "0 hour ago": [
            "часов"
        ],
        "0 minute ago": [
            "оваа минута"
        ],
        "0 month ago": [
            "овој месец"
        ],
        "0 second ago": [
            "сега"
        ],
        "0 week ago": [
            "оваа седмица"
        ],
        "0 year ago": [
            "оваа година"
        ],
        "1 day ago": [
            "вчера"
        ],
        "1 month ago": [
            "минатиот месец"
        ],
        "1 week ago": [
            "минатата седмица"
        ],
        "1 year ago": [
            "минатата година"
        ],
        "in 1 day": [
            "утре"
        ],
        "in 1 month": [
            "следниот месец"
        ],
        "in 1 week": [
            "следната седмица"
        ],
        "in 1 year": [
            "следната година"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "пред (\\d+[.,]?\\d*) ден",
            "пред (\\d+[.,]?\\d*) дена"
        ],
        "\\1 hour ago": [
            "пред (\\d+[.,]?\\d*) час",
            "пред (\\d+[.,]?\\d*) часа"
        ],
        "\\1 minute ago": [
            "пред (\\d+[.,]?\\d*) минута",
            "пред (\\d+[.,]?\\d*) минути"
        ],
        "\\1 month ago": [
            "пред (\\d+[.,]?\\d*) месец",
            "пред (\\d+[.,]?\\d*) месеци"
        ],
        "\\1 second ago": [
            "пред (\\d+[.,]?\\d*) секунда",
            "пред (\\d+[.,]?\\d*) секунди"
        ],
        "\\1 week ago": [
            "пред (\\d+[.,]?\\d*) седмица",
            "пред (\\d+[.,]?\\d*) седмици"
        ],
        "\\1 year ago": [
            "пред (\\d+[.,]?\\d*) година",
            "пред (\\d+[.,]?\\d*) години"
        ],
        "in \\1 day": [
            "за (\\d+[.,]?\\d*) ден",
            "за (\\d+[.,]?\\d*) дена"
        ],
        "in \\1 hour": [
            "за (\\d+[.,]?\\d*) час",
            "за (\\d+[.,]?\\d*) часа"
        ],
        "in \\1 minute": [
            "за (\\d+[.,]?\\d*) минута",
            "за (\\d+[.,]?\\d*) минути"
        ],
        "in \\1 month": [
            "за (\\d+[.,]?\\d*) месец",
            "за (\\d+[.,]?\\d*) месеци"
        ],
        "in \\1 second": [
            "за (\\d+[.,]?\\d*) секунда",
            "за (\\d+[.,]?\\d*) секунди"
        ],
        "in \\1 week": [
            "за (\\d+[.,]?\\d*) седмица",
            "за (\\d+[.,]?\\d*) седмици"
        ],
        "in \\1 year": [
            "за (\\d+[.,]?\\d*) година",
            "за (\\d+[.,]?\\d*) години"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "mn",
    "date_order": "YMD",
    "january": [
        "1-р сар",
        "нэгдүгээр сар"
    ],
    "february": [
        "2-р сар",
        "хоёрдугаар сар"
    ],
    "march": [
        "3-р сар",
        "гуравдугаар сар"
    ],
    "april": [
        "4-р сар",
        "дөрөвдүгээр сар"
    ],
    "may": [
        "5-р сар",
        "тавдугаар сар"
    ],
    "june": [
        "6-р сар",
        "зургадугаар сар"
    ],
    "july": [
        "7-р сар",
        "долдугаар сар"
    ],
    "august": [
        "8-р сар",
        "наймдугаар сар"
    ],
    "september": [
        "9-р сар",
        "есдүгээр сар"
    ],
    "october": [
        "10-р сар",
        "аравдугаар сар"
    ],
    "november": [
        "11-р сар",
        "арван нэгдүгээр сар"
    ],
    "december": [
        "12-р сар",
        "арван хоёрдугаар сар"
    ],
    "monday": [
        "да",
        "даваа"
    ],
    "tuesday": [
        "мя",
        "мягмар"
    ],
    "wednesday": [
        "лх",
        "лхагва"
    ],
    "thursday": [
        "пү",
        "пүрэв"
    ],
    "friday": [
        "ба",
        "баасан"
    ],
    "saturday": [
        "бя",
        "бямба"
    ],
    "sunday": [
        "ня",
        "ням"
    ],
    "am": [
        "үө"
    ],
    "pm": [
        "үх"
    ],
    "year": [
        "жил"
    ],
    "month": [
        "сар"
    ],
    "week": [
        "7х",
        "долоо хоног"
    ],
    "day": [
        "өдөр"
    ],
    "hour": [
        "ц",
        "цаг"
    ],
    "minute": [
        "мин",
        "минут"
    ],
    "second": [
        "сек",
        "секунд"
    ],
    "relative-type": {
        "0 day ago": [
            "өнөөдөр"
        ],
        "0 hour ago": [
            "энэ цаг"
        ],
        "0 minute ago": [
            "энэ минут"
        ],
        "0 month ago": [
            "энэ сар"
        ],
        "0 second ago": [
            "одоо"
        ],
        "0 week ago": [
            "энэ долоо хоног"
        ],
        "0 year ago": [
            "энэ жил"
        ],
        "1 day ago": [
            "өчигдөр"
        ],
        "1 month ago": [
            "өнгөрсөн сар"
        ],
        "1 week ago": [
            "өнгөрсөн долоо хоног"
        ],
        "1 year ago": [
            "өнгөрсөн жил"
        ],
        "in 1 day": [
            "маргааш"
        ],
        "in 1 month": [
            "ирэх сар"
        ],
        "in 1 week": [
            "ирэх долоо хоног"
        ],
        "in 1 year": [
            "ирэх жил"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) өдрийн өмнө"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) ц өмнө",
            "(\\d+[.,]?\\d*) цагийн өмнө"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) мин өмнө",
            "(\\d+[.,]?\\d*) минутын өмнө"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) сарын өмнө"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) сек өмнө",
            "(\\d+[.,]?\\d*) секундын өмнө"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) 7х-ийн өмнө"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) жилийн өмнө"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) өдрийн дараа",
            "(\\d+[.,]?\\d*) өдөрт"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) ц дараа",
            "(\\d+[.,]?\\d*) цагийн дараа"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) мин дараа",
            "(\\d+[.,]?\\d*) минутын дараа"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) сарын дараа"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) сек дараа",
            "(\\d+[.,]?\\d*) секундын дараа"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) 7х-ийн дараа"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) жилийн дараа"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import calendar
import regex as re

import pytz
from io import StringIO
from collections import OrderedDict
from datetime import datetime
from datetime import timedelta

from dateparser.utils import set_correct_day_from_settings, \
    get_last_day_of_month, get_previous_leap_year, get_next_leap_year, \
    _get_missing_parts, get_timezone_from_tz_string
from dateparser.utils.strptime import strptime


NSP_COMPATIBLE = re.compile(r'\D+')
MERIDIAN = re.compile(r'am|pm')
MICROSECOND = re.compile(r'\d{1,6}')
EIGHT_DIGIT = re.compile(r'^\d{8}$')
HOUR_MINUTE_REGEX = re.compile(r'^([0-9]|0[0-9]|1[0-9]|2[0-3]):[0-5][0-9]$')


def no_space_parser_eligibile(datestring):
    src = NSP_COMPATIBLE.search(datestring)
    if not src or ':' == src.group():
        return True
    return False


def get_unresolved_attrs(parser_object):
    attrs = ['year', 'month', 'day']
    seen = []
    unseen = []
    for attr in attrs:
        if getattr(parser_object, attr, None) is not None:
            seen.append(attr)
        else:
            unseen.append(attr)
    return seen, unseen


date_order_chart = {
    'DMY': '%d%m%y',
    'DYM': '%d%y%m',
    'MDY': '%m%d%y',
    'MYD': '%m%y%d',
    'YDM': '%y%d%m',
    'YMD': '%y%m%d',
}


def resolve_date_order(order, lst=None):

    chart_list = {
        'DMY': ['day', 'month', 'year'],
        'DYM': ['day', 'year', 'month'],
        'MDY': ['month', 'day', 'year'],
        'MYD': ['month', 'year', 'day'],
        'YDM': ['year', 'day', 'month'],
        'YMD': ['year', 'month', 'day'],
    }

    return chart_list[order] if lst else date_order_chart[order]


def _parse_absolute(datestring, settings, tz=None):
    return _parser.parse(datestring, settings, tz)


def _parse_nospaces(datestring, settings, tz=None):
    return _no_spaces_parser.parse(datestring, settings)


class _time_parser:
    time_directives = [
        '%H:%M:%S',
        '%I:%M:%S %p',
        '%H:%M',
        '%I:%M %p',
        '%I %p',
        '%H:%M:%S.%f',
        '%I:%M:%S.%f %p',
        '%H:%M %p'
    ]

    def __call__(self, timestring):
        _timestring = timestring
        for directive in self.time_directives:
            try:
                return strptime(timestring.strip(), directive).time()
            except ValueError:
                pass
        else:
            raise ValueError('%s does not seem to be a valid time string' % _timestring)


time_parser = _time_parser()


class _no_spaces_parser:
    _dateformats = [
        '%Y%m%d', '%Y%d%m', '%m%Y%d',
        '%m%d%Y', '%d%Y%m', '%d%m%Y',
        '%y%m%d', '%y%d%m', '%m%y%d',
        '%m%d%y', '%d%y%m', '%d%m%y'
    ]

    _preferred_formats = ['%Y%m%d%H%M', '%Y%m%d%H%M%S', '%Y%m%d%H%M%S.%f']

    _preferred_formats_ordered_8_digit = ['%m%d%Y', '%d%m%Y', '%Y%m%d', '%Y%d%m', '%m%Y%d', '%d%Y%m']

    _timeformats = ['%H%M%S.%f', '%H%M%S', '%H%M', '%H']

    period = {
        'day': ['%d', '%H', '%M', '%S'],
        'month': ['%m']
    }

    _default_order = resolve_date_order('MDY')

    def __init__(self, *args, **kwargs):

        self._all = (
            self._dateformats + [x + y for x in self._dateformats for y in self._timeformats] + self._timeformats
        )

        self.date_formats = {
            '%m%d%y': (
                self._preferred_formats
                + sorted(self._all, key=lambda x: x.lower().startswith('%m%d%y'), reverse=True)
            ),
            '%m%y%d': sorted(self._all, key=lambda x: x.lower().startswith('%m%y%d'), reverse=True),
            '%y%m%d': sorted(self._all, key=lambda x: x.lower().startswith('%y%m%d'), reverse=True),
            '%y%d%m': sorted(self._all, key=lambda x: x.lower().startswith('%y%d%m'), reverse=True),
            '%d%m%y': sorted(self._all, key=lambda x: x.lower().startswith('%d%m%y'), reverse=True),
            '%d%y%m': sorted(self._all, key=lambda x: x.lower().startswith('%d%y%m'), reverse=True),
        }

    @classmethod
    def _get_period(cls, format_string):
        for pname, pdrv in sorted(cls.period.items(), key=lambda x: x[0]):
            for drv in pdrv:
                if drv in format_string:
                    return pname
        else:
            return 'year'

    @classmethod
    def _find_best_matching_date(cls, datestring):
        for fmt in cls._preferred_formats_ordered_8_digit:
            try:
                dt = strptime(datestring, fmt), cls._get_period(fmt)
                if len(str(dt[0].year)) == 4:
                    return dt
            except:
                pass
        return None

    @classmethod
    def parse(cls, datestring, settings):
        if not no_space_parser_eligibile(datestring):
            raise ValueError('Unable to parse date from: %s' % datestring)

        datestring = datestring.replace(':', '')
        if not datestring:
            raise ValueError("Empty string")
        tokens = tokenizer(datestring)
        if settings.DATE_ORDER:
            order = resolve_date_order(settings.DATE_ORDER)
        else:
            order = cls._default_order
            if EIGHT_DIGIT.match(datestring):
                dt = cls._find_best_matching_date(datestring)
                if dt is not None:
                    return dt
        nsp = cls()
        ambiguous_date = None
        for token, _ in tokens.tokenize():
            for fmt in nsp.date_formats[order]:
                try:
                    dt = strptime(token, fmt), cls._get_period(fmt)
                    if len(str(dt[0].year)) < 4:
                        ambiguous_date = dt
                        continue

                    missing = _get_missing_parts(fmt)
                    _check_strict_parsing(missing, settings)
                    return dt
                except:
                    pass
        else:
            if ambiguous_date:
                return ambiguous_date
            else:
                raise ValueError('Unable to parse date from: %s' % datestring)


def _get_missing_error(missing):
    return 'Fields missing from the date string: {}'.format(', '.join(missing))


def _check_strict_parsing(missing, settings):
    if settings.STRICT_PARSING and missing:
        raise ValueError(_get_missing_error(missing))
    elif settings.REQUIRE_PARTS and missing:
        errors = [part for part in settings.REQUIRE_PARTS if part in missing]
        if errors:
            raise ValueError(_get_missing_error(errors))


class _parser:

    alpha_directives = OrderedDict([
        ('weekday', ['%A', '%a']),
        ('month', ['%B', '%b']),
    ])

    num_directives = {
        'month': ['%m'],
        'day': ['%d'],
        'year': ['%y', '%Y'],
    }

    def __init__(self, tokens, settings):
        self.settings = settings
        self.tokens = [(t[0].strip(), t[1]) for t in list(tokens)]
        self.filtered_tokens = [(t[0], t[1], i) for i, t in enumerate(self.tokens) if t[1] <= 1]

        self.unset_tokens = []

        self.day = None
        self.month = None
        self.year = None
        self.time = None

        self.auto_order = []

        self._token_day = None
        self._token_month = None
        self._token_year = None
        self._token_time = None

        self.ordered_num_directives = OrderedDict(
            (k, self.num_directives[k])
            for k in (resolve_date_order(settings.DATE_ORDER, lst=True))
        )

        skip_index = []
        skip_component = None
        skip_tokens = ["t", "year", "hour", "minute"]

        for index, token_type_original_index in enumerate(self.filtered_tokens):

            if index in skip_index:
                continue

            token, type, original_index = token_type_original_index

            if token in skip_tokens:
                continue

            if self.time is None:
                meridian_index = index + 1

                try:
                    # try case where hours and minutes are separated by a period. Example: 13.20.
                    _is_before_period = self.tokens[original_index + 1][0] == '.'
                    _is_after_period = original_index != 0 and self.tokens[original_index - 1][0] == '.'

                    if _is_before_period and not _is_after_period:
                        index_next_token = index + 1
                        next_token = self.filtered_tokens[index_next_token][0]
                        index_in_tokens_for_next_token = self.filtered_tokens[index_next_token][2]

                        next_token_is_last = index_next_token == len(self.filtered_tokens) - 1
                        if next_token_is_last or self.tokens[index_in_tokens_for_next_token + 1][0] != '.':
                            new_token = token + ':' + next_token
                            if re.match(HOUR_MINUTE_REGEX, new_token):
                                token = new_token
                                skip_index.append(index + 1)
                                meridian_index += 1
                except Exception:
                    pass

                try:
                    microsecond = MICROSECOND.search(self.filtered_tokens[index + 1][0]).group()
                    # Is after time token? raise ValueError if ':' can't be found:
                    token.index(":")
                    # Is after period? raise ValueError if '.' can't be found:
                    self.tokens[self.tokens.index((token, 0)) + 1][0].index('.')
                except:
                    microsecond = None

                if microsecond:
                    meridian_index += 1

                try:
                    meridian = MERIDIAN.search(self.filtered_tokens[meridian_index][0]).group()
                except:
                    meridian = None

                if any([':' in token, meridian, microsecond]):
                    if meridian and not microsecond:
                        self._token_time = '%s %s' % (token, meridian)
                        skip_index.append(meridian_index)
                    elif microsecond and not meridian:
                        self._token_time = '%s.%s' % (token, microsecond)
                        skip_index.append(index + 1)
                    elif meridian and microsecond:
                        self._token_time = '%s.%s %s' % (token, microsecond, meridian)
                        skip_index.append(index + 1)
                        skip_index.append(meridian_index)
                    else:
                        self._token_time = token
                    self.time = lambda: time_parser(self._token_time)
                    continue

            results = self._parse(type, token, skip_component=skip_component)
            for res in results:
                if len(token) == 4 and res[0] == 'year':
                    skip_component = 'year'
                setattr(self, *res)

        known, unknown = get_unresolved_attrs(self)
        params = {}
        for attr in known:
            params.update({attr: getattr(self, attr)})
        for attr in unknown:
            for token, type, _ in self.unset_tokens:
                if type == 0:
                    params.update({attr: int(token)})
                    setattr(self, '_token_%s' % attr, token)
                    setattr(self, attr, int(token))

    def _get_period(self):
        if self.settings.RETURN_TIME_AS_PERIOD:
            if getattr(self, 'time', None):
                return 'time'

        for period in ['time', 'day']:
            if getattr(self, period, None):
                return 'day'

        for period in ['month', 'year']:
            if getattr(self, period, None):
                return period

        if self._results():
            return 'day'

    def _get_datetime_obj(self, **params):
        try:
            return datetime(**params)
        except ValueError as e:
            error_text = e.__str__()
            error_msgs = ['day is out of range', 'day must be in']
            if error_msgs[0] in error_text or error_msgs[1] in error_text:
                if not(self._token_day or hasattr(self, '_token_weekday')):
                    # if day is not available put last day of the month
                    params['day'] = get_last_day_of_month(params['year'], params['month'])
                    return datetime(**params)
                elif not self._token_year and params['day'] == 29 and params['month'] == 2 and \
                        not calendar.isleap(params['year']):
                    # fix the year when year is not present and it is 29 of February
                    params['year'] = self._get_correct_leap_year(self.settings.PREFER_DATES_FROM, params['year'])
                    return datetime(**params)
            raise e

    def _get_correct_leap_year(self, prefer_dates_from, current_year):
        if prefer_dates_from == 'future':
            return get_next_leap_year(current_year)
        if prefer_dates_from == 'past':
            return get_previous_leap_year(current_year)

        # Default case ('current_period'): return closer leap year
        next_leap_year = get_next_leap_year(current_year)
        previous_leap_year = get_previous_leap_year(current_year)
        next_leap_year_is_closer = next_leap_year - current_year < current_year - previous_leap_year
        return next_leap_year if next_leap_year_is_closer else previous_leap_year

    def _set_relative_base(self):
        self.now = self.settings.RELATIVE_BASE
        if not self.now:
            self.now = datetime.utcnow()

    def _get_datetime_obj_params(self):
        if not self.now:
            self._set_relative_base()

        params = {
            'day': self.day or self.now.day,
            'month': self.month or self.now.month,
            'year': self.year or self.now.year,
            'hour': 0, 'minute': 0, 'second': 0, 'microsecond': 0,
        }
        return params

    def _get_date_obj(self, token, directive):
        return strptime(token, directive)

    def _results(self):
        missing = [
            field for field in ('day', 'month', 'year')
            if not getattr(self, field)
        ]
        _check_strict_parsing(missing, self.settings)
        self._set_relative_base()

        time = self.time() if self.time is not None else None
        params = self._get_datetime_obj_params()

        if time:
            params.update(dict(hour=time.hour,
                               minute=time.minute,
                               second=time.second,
                               microsecond=time.microsecond))

        return self._get_datetime_obj(**params)

    def _correct_for_time_frame(self, dateobj, tz):
        days = ['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun']

        token_weekday, _ = getattr(self, '_token_weekday', (None, None))

        if token_weekday and not(self._token_year or self._token_month or self._token_day):
            day_index = calendar.weekday(dateobj.year, dateobj.month, dateobj.day)
            day = token_weekday[:3].lower()
            steps = 0
            if 'future' in self.settings.PREFER_DATES_FROM:
                if days[day_index] == day:
                    steps = 7
                else:
                    while days[day_index] != day:
                        day_index = (day_index + 1) % 7
                        steps += 1
                delta = timedelta(days=steps)
            else:
                if days[day_index] == day:
                    if self.settings.PREFER_DATES_FROM == 'past':
                        steps = 7
                    else:
                        steps = 0
                else:
                    while days[day_index] != day:
                        day_index -= 1
                        steps += 1
                delta = timedelta(days=-steps)

            dateobj = dateobj + delta

        # NOTE: If this assert fires, self.now needs to be made offset-aware in a similar
        # way that dateobj is temporarily made offset-aware.
        assert not (self.now.tzinfo is None and dateobj.tzinfo is not None),\
            "`self.now` doesn't have `tzinfo`. Review comment in code for details."

        # Store the original dateobj values so that upon subsequent parsing everything is not
        # treated as offset-aware if offset awareness is changed.
        original_dateobj = dateobj

        # Since date comparisons must be either offset-naive or offset-aware, normalize dateobj
        # to be offset-aware if one or the other is already offset-aware.
        if self.now.tzinfo is not None and dateobj.tzinfo is None:
            dateobj = pytz.utc.localize(dateobj)

        if self.month and not self.year:
            try:
                if self.now < dateobj:
                    if self.settings.PREFER_DATES_FROM == 'past':
                        dateobj = dateobj.replace(year=dateobj.year - 1)
                else:
                    if self.settings.PREFER_DATES_FROM == 'future':
                        dateobj = dateobj.replace(year=dateobj.year + 1)
            except ValueError as e:
                if dateobj.day == 29 and dateobj.month == 2:
                    valid_year = self._get_correct_leap_year(
                        self.settings.PREFER_DATES_FROM, dateobj.year)
                    dateobj = dateobj.replace(year=valid_year)
                else:
                    raise e

        if self._token_year and len(self._token_year[0]) == 2:
            if self.now < dateobj:
                if 'past' in self.settings.PREFER_DATES_FROM:
                    dateobj = dateobj.replace(year=dateobj.year - 100)
            else:
                if 'future' in self.settings.PREFER_DATES_FROM:
                    dateobj = dateobj.replace(year=dateobj.year + 100)

        if self._token_time and not any([self._token_year,
                                         self._token_month,
                                         self._token_day,
                                         hasattr(self, '_token_weekday')]):
            # Convert dateobj to utc time to compare with self.now
            try:
                tz = tz or get_timezone_from_tz_string(self.settings.TIMEZONE)
            except pytz.UnknownTimeZoneError:
                tz = None

            if tz:
                dateobj_time = (dateobj - tz.utcoffset(dateobj)).time()
            else:
                dateobj_time = dateobj.time()
            if 'past' in self.settings.PREFER_DATES_FROM:
                if self.now.time() < dateobj_time:
                    dateobj = dateobj + timedelta(days=-1)
            if 'future' in self.settings.PREFER_DATES_FROM:
                if self.now.time() > dateobj_time:
                    dateobj = dateobj + timedelta(days=1)

        # Reset dateobj to the original value, thus removing any offset awareness that may
        # have been set earlier.
        dateobj = dateobj.replace(tzinfo=original_dateobj.tzinfo)

        return dateobj

    def _correct_for_day(self, dateobj):
        if (
            getattr(self, '_token_day', None)
            or getattr(self, '_token_weekday', None)
            or getattr(self, '_token_time', None)
        ):
            return dateobj

        dateobj = set_correct_day_from_settings(
            dateobj, self.settings, current_day=self.now.day
        )
        return dateobj

    @classmethod
    def parse(cls, datestring, settings, tz=None):
        tokens = tokenizer(datestring)
        po = cls(tokens.tokenize(), settings)
        dateobj = po._results()

        # correction for past, future if applicable
        dateobj = po._correct_for_time_frame(dateobj, tz)

        # correction for preference of day: beginning, current, end
        dateobj = po._correct_for_day(dateobj)
        period = po._get_period()

        return dateobj, period

    def _parse(self, type, token, skip_component=None):

        def set_and_return(token, type, component, dateobj, skip_date_order=False):
            if not skip_date_order:
                self.auto_order.append(component)
            setattr(self, '_token_%s' % component, (token, type))
            return [(component, getattr(dateobj, component))]

        def parse_number(token, skip_component=None):
            type = 0

            for component, directives in self.ordered_num_directives.items():
                if skip_component == component:
                    continue
                for directive in directives:
                    try:
                        do = self._get_date_obj(token, directive)
                        prev_value = getattr(self, component, None)
                        if not prev_value:
                            return set_and_return(token, type, component, do)
                        else:
                            try:
                                prev_token, prev_type = getattr(self, '_token_%s' % component)
                                if prev_type == type:
                                    do = self._get_date_obj(prev_token, directive)
                            except ValueError:
                                self.unset_tokens.append((prev_token, prev_type, component))
                                return set_and_return(token, type, component, do)
                    except ValueError:
                        pass
            else:
                raise ValueError('Unable to parse: %s' % token)

        def parse_alpha(token, skip_component=None):
            type = 1

            for component, directives in self.alpha_directives.items():
                if skip_component == component:
                    continue
                for directive in directives:
                    try:
                        do = self._get_date_obj(token, directive)
                        prev_value = getattr(self, component, None)
                        if not prev_value:
                            return set_and_return(token, type, component, do, skip_date_order=True)
                        elif component == 'month':
                            index = self.auto_order.index('month')
                            self.auto_order[index] = 'day'
                            setattr(self, '_token_day', self._token_month)
                            setattr(self, '_token_month', (token, type))
                            return [(component, getattr(do, component)), ('day', prev_value)]
                    except:
                        pass
            else:
                raise ValueError('Unable to parse: %s' % token)

        handlers = {0: parse_number, 1: parse_alpha}
        return handlers[type](token, skip_component)


class tokenizer:
    digits = '0123456789:'
    letters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'

    def _isletter(self, tkn):
        return tkn in self.letters

    def _isdigit(self, tkn):
        return tkn in self.digits

    def __init__(self, ds):
        self.instream = StringIO(ds)

    def _switch(self, chara, charb):
        if self._isdigit(chara):
            return 0, not self._isdigit(charb)

        if self._isletter(chara):
            return 1, not self._isletter(charb)

        return 2, self._isdigit(charb) or self._isletter(charb)

    def tokenize(self):
        token = ''
        EOF = False

        while not EOF:
            nextchar = self.instream.read(1)

            if not nextchar:
                EOF = True
                type, _ = self._switch(token[-1], nextchar)
                yield token, type
                return

            if token:
                type, switch = self._switch(token[-1], nextchar)

                if not switch:
                    token += nextchar
                else:
                    yield token, type
                    token = nextchar
            else:
                token += nextchar



info = {
    "name": "dav",
    "date_order": "DMY",
    "january": [
        "imb",
        "mori ghwa imbiri"
    ],
    "february": [
        "kaw",
        "mori ghwa kawi"
    ],
    "march": [
        "kad",
        "mori ghwa kadadu"
    ],
    "april": [
        "kan",
        "mori ghwa kana"
    ],
    "may": [
        "kas",
        "mori ghwa kasanu"
    ],
    "june": [
        "kar",
        "mori ghwa karandadu"
    ],
    "july": [
        "mfu",
        "mori ghwa mfungade"
    ],
    "august": [
        "mori ghwa wunyanya",
        "wun"
    ],
    "september": [
        "ike",
        "mori ghwa ikenda"
    ],
    "october": [
        "iku",
        "mori ghwa ikumi"
    ],
    "november": [
        "imw",
        "mori ghwa ikumi na imweri"
    ],
    "december": [
        "iwi",
        "mori ghwa ikumi na iwi"
    ],
    "monday": [
        "jim",
        "kuramuka jimweri"
    ],
    "tuesday": [
        "kaw",
        "kuramuka kawi"
    ],
    "wednesday": [
        "kad",
        "kuramuka kadadu"
    ],
    "thursday": [
        "kan",
        "kuramuka kana"
    ],
    "friday": [
        "kas",
        "kuramuka kasanu"
    ],
    "saturday": [
        "kifula nguwo",
        "ngu"
    ],
    "sunday": [
        "ituku ja jumwa",
        "jum"
    ],
    "am": [
        "luma lwa k"
    ],
    "pm": [
        "luma lwa p"
    ],
    "year": [
        "mwaka"
    ],
    "month": [
        "mori"
    ],
    "week": [
        "juma"
    ],
    "day": [
        "ituku"
    ],
    "hour": [
        "saa"
    ],
    "minute": [
        "dakika"
    ],
    "second": [
        "sekunde"
    ],
    "relative-type": {
        "0 day ago": [
            "idime"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "iguo"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "kesho"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "teo",
    "date_order": "DMY",
    "january": [
        "orara",
        "rar"
    ],
    "february": [
        "muk",
        "omuk"
    ],
    "march": [
        "kwa",
        "okwamg'"
    ],
    "april": [
        "dun",
        "odung'el"
    ],
    "may": [
        "mar",
        "omaruk"
    ],
    "june": [
        "mod",
        "omodok'king'ol"
    ],
    "july": [
        "jol",
        "ojola"
    ],
    "august": [
        "opedel",
        "ped"
    ],
    "september": [
        "osokosokoma",
        "sok"
    ],
    "october": [
        "otibar",
        "tib"
    ],
    "november": [
        "lab",
        "olabor"
    ],
    "december": [
        "opoo",
        "poo"
    ],
    "monday": [
        "bar",
        "nakaebarasa"
    ],
    "tuesday": [
        "aar",
        "nakaare"
    ],
    "wednesday": [
        "nakauni",
        "uni"
    ],
    "thursday": [
        "nakaung'on",
        "ung"
    ],
    "friday": [
        "kan",
        "nakakany"
    ],
    "saturday": [
        "nakasabiti",
        "sab"
    ],
    "sunday": [
        "jum",
        "nakaejuma"
    ],
    "am": [
        "taparachu"
    ],
    "pm": [
        "ebongi"
    ],
    "year": [
        "ekan"
    ],
    "month": [
        "elap"
    ],
    "week": [
        "ewiki"
    ],
    "day": [
        "aparan"
    ],
    "hour": [
        "esaa"
    ],
    "minute": [
        "idakika"
    ],
    "second": [
        "isekonde"
    ],
    "relative-type": {
        "0 day ago": [
            "lolo"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "jaan"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "moi"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {
        "teo-KE": {
            "name": "teo-KE"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "lv",
    "date_order": "DMY",
    "january": [
        "janv",
        "janvāris"
    ],
    "february": [
        "febr",
        "februāris"
    ],
    "march": [
        "marts"
    ],
    "april": [
        "apr",
        "aprīlis"
    ],
    "may": [
        "maijs"
    ],
    "june": [
        "jūn",
        "jūnijs"
    ],
    "july": [
        "jūl",
        "jūlijs"
    ],
    "august": [
        "aug",
        "augusts"
    ],
    "september": [
        "sept",
        "septembris"
    ],
    "october": [
        "okt",
        "oktobris"
    ],
    "november": [
        "nov",
        "novembris"
    ],
    "december": [
        "dec",
        "decembris"
    ],
    "monday": [
        "pirmd",
        "pirmdiena"
    ],
    "tuesday": [
        "otrd",
        "otrdiena"
    ],
    "wednesday": [
        "trešd",
        "trešdiena"
    ],
    "thursday": [
        "ceturtd",
        "ceturtdiena"
    ],
    "friday": [
        "piektd",
        "piektdiena"
    ],
    "saturday": [
        "sestd",
        "sestdiena"
    ],
    "sunday": [
        "svētd",
        "svētdiena"
    ],
    "am": [
        "priekšp",
        "priekšpusdiena",
        "priekšpusdienā"
    ],
    "pm": [
        "pēcp",
        "pēcpusd",
        "pēcpusdiena",
        "pēcpusdienā"
    ],
    "year": [
        "g",
        "gads"
    ],
    "month": [
        "mēn",
        "mēnesis"
    ],
    "week": [
        "ned",
        "nedēļa"
    ],
    "day": [
        "d",
        "diena"
    ],
    "hour": [
        "h",
        "st",
        "stundas"
    ],
    "minute": [
        "min",
        "minūtes"
    ],
    "second": [
        "s",
        "sek",
        "sekundes"
    ],
    "relative-type": {
        "0 day ago": [
            "šodien"
        ],
        "0 hour ago": [
            "šajā stundā"
        ],
        "0 minute ago": [
            "šajā minūtē"
        ],
        "0 month ago": [
            "šajā mēnesī"
        ],
        "0 second ago": [
            "tagad"
        ],
        "0 week ago": [
            "šajā nedēļā"
        ],
        "0 year ago": [
            "šajā gadā"
        ],
        "1 day ago": [
            "vakar"
        ],
        "1 month ago": [
            "pagājušajā mēnesī"
        ],
        "1 week ago": [
            "pagājušajā nedēļā"
        ],
        "1 year ago": [
            "pagājušajā gadā"
        ],
        "in 1 day": [
            "rīt"
        ],
        "in 1 month": [
            "nākamajā mēnesī"
        ],
        "in 1 week": [
            "nākamajā nedēļā"
        ],
        "in 1 year": [
            "nākamajā gadā"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "pirms (\\d+[.,]?\\d*) d",
            "pirms (\\d+[.,]?\\d*) dienas",
            "pirms (\\d+[.,]?\\d*) dienām"
        ],
        "\\1 hour ago": [
            "pirms (\\d+[.,]?\\d*) h",
            "pirms (\\d+[.,]?\\d*) st",
            "pirms (\\d+[.,]?\\d*) stundas",
            "pirms (\\d+[.,]?\\d*) stundām"
        ],
        "\\1 minute ago": [
            "pirms (\\d+[.,]?\\d*) min",
            "pirms (\\d+[.,]?\\d*) minūtes",
            "pirms (\\d+[.,]?\\d*) minūtēm"
        ],
        "\\1 month ago": [
            "pirms (\\d+[.,]?\\d*) mēn",
            "pirms (\\d+[.,]?\\d*) mēneša",
            "pirms (\\d+[.,]?\\d*) mēnešiem"
        ],
        "\\1 second ago": [
            "pirms (\\d+[.,]?\\d*) s",
            "pirms (\\d+[.,]?\\d*) sek",
            "pirms (\\d+[.,]?\\d*) sekundes",
            "pirms (\\d+[.,]?\\d*) sekundēm"
        ],
        "\\1 week ago": [
            "pirms (\\d+[.,]?\\d*) ned",
            "pirms (\\d+[.,]?\\d*) nedēļas",
            "pirms (\\d+[.,]?\\d*) nedēļām"
        ],
        "\\1 year ago": [
            "pirms (\\d+[.,]?\\d*) g",
            "pirms (\\d+[.,]?\\d*) gada",
            "pirms (\\d+[.,]?\\d*) gadiem"
        ],
        "in \\1 day": [
            "pēc (\\d+[.,]?\\d*) d",
            "pēc (\\d+[.,]?\\d*) dienas",
            "pēc (\\d+[.,]?\\d*) dienām"
        ],
        "in \\1 hour": [
            "pēc (\\d+[.,]?\\d*) h",
            "pēc (\\d+[.,]?\\d*) st",
            "pēc (\\d+[.,]?\\d*) stundas",
            "pēc (\\d+[.,]?\\d*) stundām"
        ],
        "in \\1 minute": [
            "pēc (\\d+[.,]?\\d*) min",
            "pēc (\\d+[.,]?\\d*) minūtes",
            "pēc (\\d+[.,]?\\d*) minūtēm"
        ],
        "in \\1 month": [
            "pēc (\\d+[.,]?\\d*) mēn",
            "pēc (\\d+[.,]?\\d*) mēneša",
            "pēc (\\d+[.,]?\\d*) mēnešiem"
        ],
        "in \\1 second": [
            "pēc (\\d+[.,]?\\d*) s",
            "pēc (\\d+[.,]?\\d*) sek",
            "pēc (\\d+[.,]?\\d*) sekundes",
            "pēc (\\d+[.,]?\\d*) sekundēm"
        ],
        "in \\1 week": [
            "pēc (\\d+[.,]?\\d*) ned",
            "pēc (\\d+[.,]?\\d*) nedēļas",
            "pēc (\\d+[.,]?\\d*) nedēļām"
        ],
        "in \\1 year": [
            "pēc (\\d+[.,]?\\d*) g",
            "pēc (\\d+[.,]?\\d*) gada",
            "pēc (\\d+[.,]?\\d*) gadiem"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ki",
    "date_order": "DMY",
    "january": [
        "jen",
        "njenuarĩ"
    ],
    "february": [
        "mwere wa kerĩ",
        "wkr"
    ],
    "march": [
        "mwere wa gatatũ",
        "wgt"
    ],
    "april": [
        "mwere wa kana",
        "wkn"
    ],
    "may": [
        "mwere wa gatano",
        "wtn"
    ],
    "june": [
        "mwere wa gatandatũ",
        "wtd"
    ],
    "july": [
        "mwere wa mũgwanja",
        "wmj"
    ],
    "august": [
        "mwere wa kanana",
        "wnn"
    ],
    "september": [
        "mwere wa kenda",
        "wkd"
    ],
    "october": [
        "mwere wa ikũmi",
        "wik"
    ],
    "november": [
        "mwere wa ikũmi na ũmwe",
        "wmw"
    ],
    "december": [
        "dit",
        "ndithemba"
    ],
    "monday": [
        "njumatatũ",
        "ntt"
    ],
    "tuesday": [
        "njumaine",
        "nmn"
    ],
    "wednesday": [
        "njumatana",
        "nmt"
    ],
    "thursday": [
        "aramithi",
        "art"
    ],
    "friday": [
        "njumaa",
        "nma"
    ],
    "saturday": [
        "njumamothi",
        "nmm"
    ],
    "sunday": [
        "kiumia",
        "kma"
    ],
    "am": [
        "kiroko"
    ],
    "pm": [
        "hwaĩ-inĩ"
    ],
    "year": [
        "mwaka"
    ],
    "month": [
        "mweri"
    ],
    "week": [
        "kiumia"
    ],
    "day": [
        "mũthenya"
    ],
    "hour": [
        "ithaa"
    ],
    "minute": [
        "ndagĩka"
    ],
    "second": [
        "sekunde"
    ],
    "relative-type": {
        "0 day ago": [
            "ũmũthĩ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ira"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "rũciũ"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "se",
    "date_order": "YMD",
    "january": [
        "ođđajagemánnu",
        "ođđj"
    ],
    "february": [
        "guov",
        "guovvamánnu"
    ],
    "march": [
        "njuk",
        "njukčamánnu"
    ],
    "april": [
        "cuo",
        "cuoŋománnu"
    ],
    "may": [
        "mies",
        "miessemánnu"
    ],
    "june": [
        "geas",
        "geassemánnu"
    ],
    "july": [
        "suoi",
        "suoidnemánnu"
    ],
    "august": [
        "borg",
        "borgemánnu"
    ],
    "september": [
        "čakč",
        "čakčamánnu"
    ],
    "october": [
        "golg",
        "golggotmánnu"
    ],
    "november": [
        "skáb",
        "skábmamánnu"
    ],
    "december": [
        "juov",
        "juovlamánnu"
    ],
    "monday": [
        "vuos",
        "vuossárga"
    ],
    "tuesday": [
        "maŋ",
        "maŋŋebárga"
    ],
    "wednesday": [
        "gask",
        "gaskavahkku"
    ],
    "thursday": [
        "duor",
        "duorasdat"
    ],
    "friday": [
        "bear",
        "bearjadat"
    ],
    "saturday": [
        "láv",
        "lávvardat"
    ],
    "sunday": [
        "sotn",
        "sotnabeaivi"
    ],
    "am": [
        "ib",
        "iđitbeaivet",
        "iđitbeaivi"
    ],
    "pm": [
        "eahketbeaivet",
        "eahketbeaivi",
        "eb"
    ],
    "year": [
        "jáhki"
    ],
    "month": [
        "mánnu"
    ],
    "week": [
        "váhkku"
    ],
    "day": [
        "beaivi"
    ],
    "hour": [
        "diibmu"
    ],
    "minute": [
        "minuhtta"
    ],
    "second": [
        "sekunda"
    ],
    "relative-type": {
        "0 day ago": [
            "odne"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ikte"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "ihttin"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) jándor árat",
            "(\\d+[.,]?\\d*) jándora árat"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) diibmu árat",
            "(\\d+[.,]?\\d*) diibmur árat"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) minuhta árat",
            "(\\d+[.,]?\\d*) minuhtta árat"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) mánotbadji árat"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) sekunda árat",
            "(\\d+[.,]?\\d*) sekundda árat"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) vahkku árat",
            "(\\d+[.,]?\\d*) vahku árat"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) jahki árat",
            "(\\d+[.,]?\\d*) jahkki árat"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) jándor maŋŋilit",
            "(\\d+[.,]?\\d*) jándora maŋŋilit"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) diibmu maŋŋilit",
            "(\\d+[.,]?\\d*) diibmur maŋŋilit"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) minuhta maŋŋilit",
            "(\\d+[.,]?\\d*) minuhtta maŋŋilit"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) mánotbadji maŋŋilit"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) sekunda maŋŋilit",
            "(\\d+[.,]?\\d*) sekundda maŋŋilit"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) vahkku maŋŋilit",
            "(\\d+[.,]?\\d*) vahku maŋŋilit"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) jahki maŋŋilit",
            "(\\d+[.,]?\\d*) jahkki maŋŋilit"
        ]
    },
    "locale_specific": {
        "se-FI": {
            "name": "se-FI",
            "monday": [
                "vuossárgga"
            ],
            "tuesday": [
                "maŋŋebárgga"
            ],
            "wednesday": [
                "gaskavahku"
            ],
            "thursday": [
                "duorastaga"
            ],
            "friday": [
                "bearjadaga"
            ],
            "saturday": [
                "lávvardaga"
            ],
            "year": [
                "j",
                "jahki"
            ],
            "week": [
                "vahkku"
            ],
            "relative-type": {
                "0 year ago": [
                    "dán jagi"
                ],
                "1 year ago": [
                    "mannan jagi"
                ],
                "in 1 year": [
                    "boahtte jagi"
                ]
            },
            "relative-type-regex": {
                "\\1 year ago": [
                    "(\\d+[.,]?\\d*) jagi árat"
                ],
                "in \\1 year": [
                    "(\\d+[.,]?\\d*) jagi siste"
                ]
            }
        },
        "se-SE": {
            "name": "se-SE"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "mua",
    "date_order": "DMY",
    "january": [
        "flo",
        "fĩi loo"
    ],
    "february": [
        "cla",
        "cokcwaklaŋne"
    ],
    "march": [
        "cki",
        "cokcwaklii"
    ],
    "april": [
        "fmf",
        "fĩi marfoo"
    ],
    "may": [
        "mad",
        "madǝǝuutǝbijaŋ"
    ],
    "june": [
        "mamǝŋgwãafahbii",
        "mbi"
    ],
    "july": [
        "mamǝŋgwãalii",
        "mli"
    ],
    "august": [
        "madǝmbii",
        "mam"
    ],
    "september": [
        "fde",
        "fĩi dǝɓlii"
    ],
    "october": [
        "fmu",
        "fĩi mundaŋ"
    ],
    "november": [
        "fgw",
        "fĩi gwahlle"
    ],
    "december": [
        "fyu",
        "fĩi yuru"
    ],
    "monday": [
        "cla",
        "comlaaɗii"
    ],
    "tuesday": [
        "comzyiiɗii",
        "czi"
    ],
    "wednesday": [
        "cko",
        "comkolle"
    ],
    "thursday": [
        "cka",
        "comkaldǝɓlii"
    ],
    "friday": [
        "cga",
        "comgaisuu"
    ],
    "saturday": [
        "comzyeɓsuu",
        "cze"
    ],
    "sunday": [
        "com'yakke",
        "cya"
    ],
    "am": [
        "comme"
    ],
    "pm": [
        "lilli"
    ],
    "year": [
        "syii"
    ],
    "month": [
        "fĩi"
    ],
    "week": [
        "luma"
    ],
    "day": [
        "zah'nane/ comme"
    ],
    "hour": [
        "cok comme"
    ],
    "minute": [
        "cok comme ma laŋne"
    ],
    "second": [
        "cok comme ma laŋ tǝ biŋ"
    ],
    "relative-type": {
        "0 day ago": [
            "tǝ'nahko"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "tǝsoo"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tǝ'nane"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "cgg",
    "date_order": "DMY",
    "january": [
        "kbz",
        "okwokubanza"
    ],
    "february": [
        "kbr",
        "okwakabiri"
    ],
    "march": [
        "kst",
        "okwakashatu"
    ],
    "april": [
        "kkn",
        "okwakana"
    ],
    "may": [
        "ktn",
        "okwakataana"
    ],
    "june": [
        "kmk",
        "okwamukaaga"
    ],
    "july": [
        "kms",
        "okwamushanju"
    ],
    "august": [
        "kmn",
        "okwamunaana"
    ],
    "september": [
        "kmw",
        "okwamwenda"
    ],
    "october": [
        "kkm",
        "okwaikumi"
    ],
    "november": [
        "knk",
        "okwaikumi na kumwe"
    ],
    "december": [
        "knb",
        "okwaikumi na ibiri"
    ],
    "monday": [
        "ork",
        "orwokubanza"
    ],
    "tuesday": [
        "okb",
        "orwakabiri"
    ],
    "wednesday": [
        "oks",
        "orwakashatu"
    ],
    "thursday": [
        "okn",
        "orwakana"
    ],
    "friday": [
        "okt",
        "orwakataano"
    ],
    "saturday": [
        "omk",
        "orwamukaaga"
    ],
    "sunday": [
        "san",
        "sande"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "omwaka"
    ],
    "month": [
        "omwezi"
    ],
    "week": [
        "esande"
    ],
    "day": [
        "eizooba"
    ],
    "hour": [
        "shaaha"
    ],
    "minute": [
        "edakiika"
    ],
    "second": [
        "obucweka/esekendi"
    ],
    "relative-type": {
        "0 day ago": [
            "erizooba"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "nyomwabazyo"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "nyenkyakare"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "en",
    "date_order": "MDY",
    "january": [
        "jan",
        "january"
    ],
    "february": [
        "feb",
        "february"
    ],
    "march": [
        "mar",
        "march"
    ],
    "april": [
        "apr",
        "april"
    ],
    "may": [
        "may"
    ],
    "june": [
        "jun",
        "june"
    ],
    "july": [
        "jul",
        "july"
    ],
    "august": [
        "aug",
        "august"
    ],
    "september": [
        "sep",
        "september",
        "sept"
    ],
    "october": [
        "oct",
        "october"
    ],
    "november": [
        "nov",
        "november"
    ],
    "december": [
        "dec",
        "december"
    ],
    "monday": [
        "mon",
        "monday"
    ],
    "tuesday": [
        "tue",
        "tuesday",
        "Tues"
    ],
    "wednesday": [
        "wed",
        "wednesday"
    ],
    "thursday": [
        "thu",
        "thursday"
    ],
    "friday": [
        "fri",
        "friday"
    ],
    "saturday": [
        "sat",
        "saturday"
    ],
    "sunday": [
        "sun",
        "sunday"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "year",
        "yr",
        "y",
        "years"
    ],
    "month": [
        "mo",
        "month",
        "months"
    ],
    "week": [
        "week",
        "wk",
        "weeks"
    ],
    "day": [
        "day",
        "d",
        "days"
    ],
    "hour": [
        "hour",
        "hr",
        "h",
        "hours",
        "hrs"
    ],
    "minute": [
        "min",
        "minute",
        "m",
        "minutes",
        "mins"
    ],
    "second": [
        "sec",
        "second",
        "s",
        "seconds",
        "secs"
    ],
    "relative-type": {
        "0 day ago": [
            "today",
            "till date"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this mo",
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week",
            "this wk"
        ],
        "0 year ago": [
            "this year",
            "this yr"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last mo",
            "last month"
        ],
        "1 week ago": [
            "last week",
            "last wk"
        ],
        "1 year ago": [
            "last year",
            "last yr"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next mo",
            "next month"
        ],
        "in 1 week": [
            "next week",
            "next wk"
        ],
        "in 1 year": [
            "next year",
            "next yr"
        ],
        "2 day ago": [
            "day before yesterday"
        ],
        "1 decade ago": [
            "last decade",
            "this decade"
        ],
        "in 1 decade": [
            "next decade"
        ],
        "in 2 day": [
            "day after tomorrow"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) day ago",
            "(\\d+[.,]?\\d*) days ago"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) hour ago",
            "(\\d+[.,]?\\d*) hours ago",
            "(\\d+[.,]?\\d*) hr ago"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) min ago",
            "(\\d+[.,]?\\d*) minute ago",
            "(\\d+[.,]?\\d*) minutes ago"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) mo ago",
            "(\\d+[.,]?\\d*) month ago",
            "(\\d+[.,]?\\d*) months ago"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) sec ago",
            "(\\d+[.,]?\\d*) second ago",
            "(\\d+[.,]?\\d*) seconds ago"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) week ago",
            "(\\d+[.,]?\\d*) weeks ago",
            "(\\d+[.,]?\\d*) wk ago"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) year ago",
            "(\\d+[.,]?\\d*) years ago",
            "(\\d+[.,]?\\d*) yr ago"
        ],
        "in \\1 day": [
            "in (\\d+[.,]?\\d*) day",
            "in (\\d+[.,]?\\d*) days"
        ],
        "in \\1 hour": [
            "in (\\d+[.,]?\\d*) hour",
            "in (\\d+[.,]?\\d*) hours",
            "in (\\d+[.,]?\\d*) hr"
        ],
        "in \\1 minute": [
            "in (\\d+[.,]?\\d*) min",
            "in (\\d+[.,]?\\d*) minute",
            "in (\\d+[.,]?\\d*) minutes"
        ],
        "in \\1 month": [
            "in (\\d+[.,]?\\d*) mo",
            "in (\\d+[.,]?\\d*) month",
            "in (\\d+[.,]?\\d*) months"
        ],
        "in \\1 second": [
            "in (\\d+[.,]?\\d*) sec",
            "in (\\d+[.,]?\\d*) second",
            "in (\\d+[.,]?\\d*) seconds"
        ],
        "in \\1 week": [
            "in (\\d+[.,]?\\d*) week",
            "in (\\d+[.,]?\\d*) weeks",
            "in (\\d+[.,]?\\d*) wk"
        ],
        "in \\1 year": [
            "in (\\d+[.,]?\\d*) year",
            "in (\\d+[.,]?\\d*) years",
            "in (\\d+[.,]?\\d*) yr"
        ],
        "in \\1 decade": [
            "in (\\d+[.,]?\\d*) decades?"
        ],
        "\\1 decade ago": [
            "(\\d+[.,]?\\d*) decades? ago"
        ]
    },
    "locale_specific": {
        "en-001": {
            "name": "en-001",
            "date_order": "DMY"
        },
        "en-150": {
            "name": "en-150",
            "date_order": "DMY"
        },
        "en-AG": {
            "name": "en-AG",
            "date_order": "DMY"
        },
        "en-AI": {
            "name": "en-AI",
            "date_order": "DMY"
        },
        "en-AS": {
            "name": "en-AS"
        },
        "en-AT": {
            "name": "en-AT",
            "date_order": "DMY"
        },
        "en-AU": {
            "name": "en-AU",
            "date_order": "DMY",
            "hour": [
                "h"
            ]
        },
        "en-BB": {
            "name": "en-BB",
            "date_order": "DMY"
        },
        "en-BE": {
            "name": "en-BE",
            "date_order": "DMY"
        },
        "en-BI": {
            "name": "en-BI"
        },
        "en-BM": {
            "name": "en-BM",
            "date_order": "DMY"
        },
        "en-BS": {
            "name": "en-BS",
            "date_order": "DMY"
        },
        "en-BW": {
            "name": "en-BW",
            "date_order": "DMY"
        },
        "en-BZ": {
            "name": "en-BZ",
            "date_order": "DMY"
        },
        "en-CA": {
            "name": "en-CA",
            "date_order": "YMD",
            "relative-type-regex": {
                "\\1 hour ago": [
                    "(\\d+[.,]?\\d*) hrs ago"
                ],
                "\\1 minute ago": [
                    "(\\d+[.,]?\\d*) mins ago"
                ],
                "\\1 month ago": [
                    "(\\d+[.,]?\\d*) mos ago"
                ],
                "\\1 second ago": [
                    "(\\d+[.,]?\\d*) secs ago"
                ],
                "\\1 week ago": [
                    "(\\d+[.,]?\\d*) wks ago"
                ],
                "\\1 year ago": [
                    "(\\d+[.,]?\\d*) yrs ago"
                ],
                "in \\1 hour": [
                    "in (\\d+[.,]?\\d*) hrs"
                ],
                "in \\1 minute": [
                    "in (\\d+[.,]?\\d*) mins"
                ],
                "in \\1 month": [
                    "in (\\d+[.,]?\\d*) mos"
                ],
                "in \\1 second": [
                    "in (\\d+[.,]?\\d*) secs"
                ],
                "in \\1 week": [
                    "in (\\d+[.,]?\\d*) wks"
                ],
                "in \\1 year": [
                    "in (\\d+[.,]?\\d*) yrs"
                ]
            }
        },
        "en-CC": {
            "name": "en-CC",
            "date_order": "DMY"
        },
        "en-CH": {
            "name": "en-CH",
            "date_order": "DMY"
        },
        "en-CK": {
            "name": "en-CK",
            "date_order": "DMY"
        },
        "en-CM": {
            "name": "en-CM",
            "date_order": "DMY"
        },
        "en-CX": {
            "name": "en-CX",
            "date_order": "DMY"
        },
        "en-CY": {
            "name": "en-CY",
            "date_order": "DMY"
        },
        "en-DE": {
            "name": "en-DE",
            "date_order": "DMY"
        },
        "en-DG": {
            "name": "en-DG",
            "date_order": "DMY"
        },
        "en-DK": {
            "name": "en-DK",
            "date_order": "DMY"
        },
        "en-DM": {
            "name": "en-DM",
            "date_order": "DMY"
        },
        "en-ER": {
            "name": "en-ER",
            "date_order": "DMY"
        },
        "en-FI": {
            "name": "en-FI",
            "date_order": "DMY"
        },
        "en-FJ": {
            "name": "en-FJ",
            "date_order": "DMY"
        },
        "en-FK": {
            "name": "en-FK",
            "date_order": "DMY"
        },
        "en-FM": {
            "name": "en-FM",
            "date_order": "DMY"
        },
        "en-GB": {
            "name": "en-GB",
            "date_order": "DMY"
        },
        "en-GD": {
            "name": "en-GD",
            "date_order": "DMY"
        },
        "en-GG": {
            "name": "en-GG",
            "date_order": "DMY"
        },
        "en-GH": {
            "name": "en-GH",
            "date_order": "DMY"
        },
        "en-GI": {
            "name": "en-GI",
            "date_order": "DMY"
        },
        "en-GM": {
            "name": "en-GM",
            "date_order": "DMY"
        },
        "en-GU": {
            "name": "en-GU"
        },
        "en-GY": {
            "name": "en-GY",
            "date_order": "DMY"
        },
        "en-HK": {
            "name": "en-HK",
            "date_order": "DMY"
        },
        "en-IE": {
            "name": "en-IE",
            "date_order": "DMY"
        },
        "en-IL": {
            "name": "en-IL",
            "date_order": "DMY"
        },
        "en-IM": {
            "name": "en-IM",
            "date_order": "DMY"
        },
        "en-IN": {
            "name": "en-IN",
            "date_order": "DMY"
        },
        "en-IO": {
            "name": "en-IO",
            "date_order": "DMY"
        },
        "en-JE": {
            "name": "en-JE",
            "date_order": "DMY"
        },
        "en-JM": {
            "name": "en-JM",
            "date_order": "DMY"
        },
        "en-KE": {
            "name": "en-KE",
            "date_order": "DMY"
        },
        "en-KI": {
            "name": "en-KI",
            "date_order": "DMY"
        },
        "en-KN": {
            "name": "en-KN",
            "date_order": "DMY"
        },
        "en-KY": {
            "name": "en-KY",
            "date_order": "DMY"
        },
        "en-LC": {
            "name": "en-LC",
            "date_order": "DMY"
        },
        "en-LR": {
            "name": "en-LR",
            "date_order": "DMY"
        },
        "en-LS": {
            "name": "en-LS",
            "date_order": "DMY"
        },
        "en-MG": {
            "name": "en-MG",
            "date_order": "DMY"
        },
        "en-MH": {
            "name": "en-MH"
        },
        "en-MO": {
            "name": "en-MO",
            "date_order": "DMY"
        },
        "en-MP": {
            "name": "en-MP"
        },
        "en-MS": {
            "name": "en-MS",
            "date_order": "DMY"
        },
        "en-MT": {
            "name": "en-MT",
            "date_order": "DMY"
        },
        "en-MU": {
            "name": "en-MU",
            "date_order": "DMY"
        },
        "en-MW": {
            "name": "en-MW",
            "date_order": "DMY"
        },
        "en-MY": {
            "name": "en-MY",
            "date_order": "DMY"
        },
        "en-NA": {
            "name": "en-NA",
            "date_order": "DMY"
        },
        "en-NF": {
            "name": "en-NF",
            "date_order": "DMY"
        },
        "en-NG": {
            "name": "en-NG",
            "date_order": "DMY"
        },
        "en-NL": {
            "name": "en-NL",
            "date_order": "DMY"
        },
        "en-NR": {
            "name": "en-NR",
            "date_order": "DMY"
        },
        "en-NU": {
            "name": "en-NU",
            "date_order": "DMY"
        },
        "en-NZ": {
            "name": "en-NZ",
            "date_order": "DMY"
        },
        "en-PG": {
            "name": "en-PG",
            "date_order": "DMY"
        },
        "en-PH": {
            "name": "en-PH",
            "date_order": "DMY"
        },
        "en-PK": {
            "name": "en-PK",
            "date_order": "DMY"
        },
        "en-PN": {
            "name": "en-PN",
            "date_order": "DMY"
        },
        "en-PR": {
            "name": "en-PR"
        },
        "en-PW": {
            "name": "en-PW",
            "date_order": "DMY"
        },
        "en-RW": {
            "name": "en-RW",
            "date_order": "DMY"
        },
        "en-SB": {
            "name": "en-SB",
            "date_order": "DMY"
        },
        "en-SC": {
            "name": "en-SC",
            "date_order": "DMY"
        },
        "en-SD": {
            "name": "en-SD",
            "date_order": "DMY"
        },
        "en-SE": {
            "name": "en-SE",
            "date_order": "YMD"
        },
        "en-SG": {
            "name": "en-SG",
            "date_order": "DMY",
            "month": [
                "mth"
            ],
            "relative-type": {
                "0 month ago": [
                    "this mth"
                ],
                "1 month ago": [
                    "last mth"
                ],
                "in 1 month": [
                    "next mth"
                ]
            },
            "relative-type-regex": {
                "\\1 month ago": [
                    "(\\d+[.,]?\\d*) mth ago"
                ],
                "in \\1 month": [
                    "in (\\d+[.,]?\\d*) mth"
                ]
            }
        },
        "en-SH": {
            "name": "en-SH",
            "date_order": "DMY"
        },
        "en-SI": {
            "name": "en-SI",
            "date_order": "DMY"
        },
        "en-SL": {
            "name": "en-SL",
            "date_order": "DMY"
        },
        "en-SS": {
            "name": "en-SS",
            "date_order": "DMY"
        },
        "en-SX": {
            "name": "en-SX",
            "date_order": "DMY"
        },
        "en-SZ": {
            "name": "en-SZ",
            "date_order": "DMY"
        },
        "en-TC": {
            "name": "en-TC",
            "date_order": "DMY"
        },
        "en-TK": {
            "name": "en-TK",
            "date_order": "DMY"
        },
        "en-TO": {
            "name": "en-TO",
            "date_order": "DMY"
        },
        "en-TT": {
            "name": "en-TT",
            "date_order": "DMY"
        },
        "en-TV": {
            "name": "en-TV",
            "date_order": "DMY"
        },
        "en-TZ": {
            "name": "en-TZ",
            "date_order": "DMY"
        },
        "en-UG": {
            "name": "en-UG",
            "date_order": "DMY"
        },
        "en-UM": {
            "name": "en-UM"
        },
        "en-VC": {
            "name": "en-VC",
            "date_order": "DMY"
        },
        "en-VG": {
            "name": "en-VG",
            "date_order": "DMY"
        },
        "en-VI": {
            "name": "en-VI"
        },
        "en-VU": {
            "name": "en-VU",
            "date_order": "DMY"
        },
        "en-WS": {
            "name": "en-WS",
            "date_order": "DMY"
        },
        "en-ZA": {
            "name": "en-ZA",
            "date_order": "YMD"
        },
        "en-ZM": {
            "name": "en-ZM",
            "date_order": "DMY"
        },
        "en-ZW": {
            "name": "en-ZW",
            "date_order": "DMY"
        }
    },
    "skip": [
        "about",
        "ad",
        "and",
        "at",
        "by",
        "just",
        "m",
        "nd",
        "of",
        "on",
        "rd",
        "st",
        "th",
        "the",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "pertain": [
        "of"
    ],
    "sentence_splitter_group": 1,
    "decade": [
        "decade",
        "decades"
    ],
    "ago": [
        "ago",
        "before"
    ],
    "in": [
        "in",
        "from now",
        "after"
    ],
    "simplifications": [
        {
            "an": "1"
        },
        {
            "a": "1"
        },
        {
            "(?:12\\s+)?noon": "12:00"
        },
        {
            "(?:12\\s+)?midnight": "00:00"
        },
        {
            "(\\d+[.,]?\\d*)h(\\d+[.,]?\\d*)m?": "\\1:\\2"
        },
        {
            "(?<=from\\s+)now": "in"
        },
        {
            "less than 1 minute ago": "45 second ago"
        },
        {
            "(\\d+[.,]?\\d*) (decade|year|month|week|day|hour|minute|second)s? later": "in \\1 \\2"
        },
        {
            "one": "1"
        },
        {
            "two": "2"
        },
        {
            "three": "3"
        },
        {
            "four": "4"
        },
        {
            "five": "5"
        },
        {
            "six": "6"
        },
        {
            "seven": "7"
        },
        {
            "eight": "8"
        },
        {
            "nine": "9"
        },
        {
            "ten": "10"
        },
        {
            "eleven": "11"
        },
        {
            "twelve": "12"
        }
    ]
}



default_parsers = [
    'timestamp',
    'relative-time',
    'custom-formats',
    'absolute-time',
]

settings = {
    # Date order
    'DATE_ORDER': 'MDY',
    'PREFER_LOCALE_DATE_ORDER': True,

    # Timezone related
    'TIMEZONE': 'local',
    'TO_TIMEZONE': False,
    'RETURN_AS_TIMEZONE_AWARE': 'default',

    # Incomplete dates
    'PREFER_DAY_OF_MONTH': 'current',
    'PREFER_DATES_FROM': 'current_period',
    'RELATIVE_BASE': False,
    'STRICT_PARSING': False,
    'REQUIRE_PARTS': [],

    # Language detection
    'SKIP_TOKENS': ['t'],
    'NORMALIZE': True,
    'DEFAULT_LANGUAGES': [],

    # Optional language detection
    'LANGUAGE_DETECTION_CONFIDENCE_THRESHOLD': 0.5,

    # Other settings
    'RETURN_TIME_AS_PERIOD': False,
    'PARSERS': default_parsers,
    'CACHE_SIZE_LIMIT': 1000,
}



info = {
    "name": "sn",
    "date_order": "YMD",
    "january": [
        "ndi",
        "ndira"
    ],
    "february": [
        "kuk",
        "kukadzi"
    ],
    "march": [
        "kur",
        "kurume"
    ],
    "april": [
        "kub",
        "kubvumbi"
    ],
    "may": [
        "chivabvu",
        "chv"
    ],
    "june": [
        "chikumi",
        "chk"
    ],
    "july": [
        "chg",
        "chikunguru"
    ],
    "august": [
        "nya",
        "nyamavhuvhu"
    ],
    "september": [
        "gun",
        "gunyana"
    ],
    "october": [
        "gum",
        "gumiguru"
    ],
    "november": [
        "mbu",
        "mbudzi"
    ],
    "december": [
        "zvi",
        "zvita"
    ],
    "monday": [
        "muv",
        "muvhuro"
    ],
    "tuesday": [
        "chipiri",
        "chp"
    ],
    "wednesday": [
        "chitatu",
        "cht"
    ],
    "thursday": [
        "china",
        "chn"
    ],
    "friday": [
        "chishanu",
        "chs"
    ],
    "saturday": [
        "mug",
        "mugovera"
    ],
    "sunday": [
        "svo",
        "svondo"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "gore"
    ],
    "month": [
        "mwedzi"
    ],
    "week": [
        "vhiki"
    ],
    "day": [
        "zuva"
    ],
    "hour": [
        "awa"
    ],
    "minute": [
        "mineti"
    ],
    "second": [
        "sekondi"
    ],
    "relative-type": {
        "0 day ago": [
            "nhasi"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "nezuro"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "mangwana"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "eu",
    "date_order": "YMD",
    "january": [
        "urt",
        "urtarrila"
    ],
    "february": [
        "ots",
        "otsaila"
    ],
    "march": [
        "mar",
        "martxoa"
    ],
    "april": [
        "api",
        "apirila"
    ],
    "may": [
        "mai",
        "maiatza"
    ],
    "june": [
        "eka",
        "ekaina"
    ],
    "july": [
        "uzt",
        "uztaila"
    ],
    "august": [
        "abu",
        "abuztua"
    ],
    "september": [
        "ira",
        "iraila"
    ],
    "october": [
        "urr",
        "urria"
    ],
    "november": [
        "aza",
        "azaroa"
    ],
    "december": [
        "abe",
        "abendua"
    ],
    "monday": [
        "al",
        "astelehena"
    ],
    "tuesday": [
        "ar",
        "asteartea"
    ],
    "wednesday": [
        "asteazkena",
        "az"
    ],
    "thursday": [
        "og",
        "osteguna"
    ],
    "friday": [
        "or",
        "ostirala"
    ],
    "saturday": [
        "larunbata",
        "lr"
    ],
    "sunday": [
        "ig",
        "igandea"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "urtea"
    ],
    "month": [
        "hil",
        "hilabetea"
    ],
    "week": [
        "ast",
        "astea"
    ],
    "day": [
        "eg",
        "eguna"
    ],
    "hour": [
        "h",
        "ordua"
    ],
    "minute": [
        "min",
        "minutua"
    ],
    "second": [
        "s",
        "segundoa"
    ],
    "relative-type": {
        "0 day ago": [
            "gaur"
        ],
        "0 hour ago": [
            "ordu honetan"
        ],
        "0 minute ago": [
            "minutu honetan"
        ],
        "0 month ago": [
            "hilabete hau"
        ],
        "0 second ago": [
            "orain"
        ],
        "0 week ago": [
            "aste hau"
        ],
        "0 year ago": [
            "aurten"
        ],
        "1 day ago": [
            "atzo"
        ],
        "1 month ago": [
            "aurreko hilabetea"
        ],
        "1 week ago": [
            "aurreko astea"
        ],
        "1 year ago": [
            "aurreko urtea"
        ],
        "in 1 day": [
            "bihar"
        ],
        "in 1 month": [
            "hurrengo hilabetea"
        ],
        "in 1 week": [
            "hurrengo astea"
        ],
        "in 1 year": [
            "hurrengo urtea"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "duela (\\d+[.,]?\\d*) egun"
        ],
        "\\1 hour ago": [
            "duela (\\d+[.,]?\\d*) ordu"
        ],
        "\\1 minute ago": [
            "duela (\\d+[.,]?\\d*) minutu"
        ],
        "\\1 month ago": [
            "duela (\\d+[.,]?\\d*) hilabete"
        ],
        "\\1 second ago": [
            "duela (\\d+[.,]?\\d*) segundo"
        ],
        "\\1 week ago": [
            "duela (\\d+[.,]?\\d*) aste"
        ],
        "\\1 year ago": [
            "duela (\\d+[.,]?\\d*) urte"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) egun barru"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) ordu barru"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) minutu barru"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) hilabete barru"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) segundo barru"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) aste barru"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) urte barru"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from itertools import chain

import regex as re
from collections import OrderedDict

from dateutil import parser

from dateparser.timezone_parser import pop_tz_offset_from_string, word_is_tz
from dateparser.utils import normalize_unicode, combine_dicts

from .dictionary import Dictionary, NormalizedDictionary, ALWAYS_KEEP_TOKENS

NUMERAL_PATTERN = re.compile(r'(\d+)', re.U)


class Locale:
    """
    Class that deals with applicability and translation from a locale.

    :param shortname:
        A locale code, e.g. 'fr-PF', 'qu-EC', 'af-NA'.
    :type shortname: str

    :param language_info:
        Language info (translation data) of the language the locale belongs to.
    :type language_info: dict

    :return: A Locale instance
    """

    _dictionary = None
    _normalized_dictionary = None
    _simplifications = None
    _normalized_simplifications = None
    _splitters = None
    _wordchars = None
    _relative_translations = None
    _normalized_relative_translations = None
    _abbreviations = None
    _split_dictionary = None
    _wordchars_for_detection = None

    def __init__(self, shortname, language_info):
        self.shortname = shortname
        locale_specific_info = language_info.get("locale_specific", {}).get(shortname, {})
        self.info = combine_dicts(language_info, locale_specific_info)
        self.info.pop("locale_specific", None)

    def is_applicable(self, date_string, strip_timezone=False, settings=None):
        """
        Check if the locale is applicable to translate date string.

        :param date_string:
            A string representing date and/or time in a recognizably valid format.
        :type date_string: str

        :param strip_timezone:
            If True, timezone is stripped from date string.
        :type strip_timezone: bool

        :return: boolean value representing if the locale is applicable for the date string or not.
        """
        if strip_timezone:
            date_string, _ = pop_tz_offset_from_string(date_string, as_offset=False)

        date_string = self._translate_numerals(date_string)
        if settings.NORMALIZE:
            date_string = normalize_unicode(date_string)
        date_string = self._simplify(date_string, settings=settings)
        dictionary = self._get_dictionary(settings)
        date_tokens = dictionary.split(date_string)
        return dictionary.are_tokens_valid(date_tokens)

    def count_applicability(self, text, strip_timezone=False, settings=None):
        if strip_timezone:
            text, _ = pop_tz_offset_from_string(text, as_offset=False)

        text = self._simplify(text, settings=settings)
        sentences = self._sentence_split(text, settings=settings)
        tokens = []
        for sent in sentences:
            tokens.extend(self._split(sent, keep_formatting=False, settings=settings))
        return self._count_words_present_in_the_dictionary(tokens, settings)

    def _count_words_present_in_the_dictionary(self, words, settings=None):
        dictionary = self.clean_dictionary(self._get_split_dictionary(settings=settings))
        dict_cnt = 0
        skip_cnt = 0
        for word in set(words):
            if word in dictionary:
                if dictionary[word]:
                    dict_cnt += 1
                else:
                    skip_cnt += 1
            elif word.isdigit():
                skip_cnt += 1
        return [dict_cnt, skip_cnt]

    @staticmethod
    def clean_dictionary(dictionary, threshold=2):
        del_keys = []
        for key in dictionary:
            if len(key) < threshold:
                del_keys.append(key)
        for del_key in del_keys:
            del dictionary[del_key]
        return dictionary

    def translate(self, date_string, keep_formatting=False, settings=None):
        """
        Translate the date string to its English equivalent.

        :param date_string:
            A string representing date and/or time in a recognizably valid format.
        :type date_string: str

        :param keep_formatting:
            If True, retain formatting of the date string after translation.
        :type keep_formatting: bool

        :return: translated date string.
        """
        date_string = self._translate_numerals(date_string)
        if settings.NORMALIZE:
            date_string = normalize_unicode(date_string)
        date_string = self._simplify(date_string, settings=settings)
        dictionary = self._get_dictionary(settings)
        date_string_tokens = dictionary.split(date_string, keep_formatting)

        relative_translations = self._get_relative_translations(settings=settings)

        for i, word in enumerate(date_string_tokens):
            word = word.lower()
            for pattern, replacement in relative_translations.items():
                if pattern.match(word):
                    date_string_tokens[i] = pattern.sub(replacement, word)
                    break
            else:
                if word in dictionary:
                    fallback = word if keep_formatting and not word.isalpha() else ''
                    date_string_tokens[i] = dictionary[word] or fallback
        if "in" in date_string_tokens:
            date_string_tokens = self._clear_future_words(date_string_tokens)

        return self._join(list(filter(bool, date_string_tokens)),
                          separator="" if keep_formatting else " ", settings=settings)

    def _translate_numerals(self, date_string):
        date_string_tokens = NUMERAL_PATTERN.split(date_string)
        for i, token in enumerate(date_string_tokens):
            if token.isdecimal():
                date_string_tokens[i] = str(int(token)).zfill(len(token))
        return ''.join(date_string_tokens)

    def _get_relative_translations(self, settings=None):
        if settings.NORMALIZE:
            if self._normalized_relative_translations is None:
                self._normalized_relative_translations = (
                    self._generate_relative_translations(normalize=True))
            return self._normalized_relative_translations
        else:
            if self._relative_translations is None:
                self._relative_translations = self._generate_relative_translations(normalize=False)
            return self._relative_translations

    def _generate_relative_translations(self, normalize=False):
        relative_translations = self.info.get('relative-type-regex', {})
        relative_dictionary = OrderedDict()
        for key, value in relative_translations.items():
            if normalize:
                value = list(map(normalize_unicode, value))
            pattern = '|'.join(sorted(value, key=len, reverse=True))
            pattern = pattern.replace(r'(\d+', r'(?P<n>\d+')
            pattern = re.compile(r'^(?:{})$'.format(pattern), re.UNICODE | re.IGNORECASE)
            relative_dictionary[pattern] = key
        return relative_dictionary

    def translate_search(self, search_string, settings=None):
        dashes = ['-', '——', '—', '～']
        word_joint_unsupported_languages = ["zh", "ja"]
        sentences = self._sentence_split(search_string, settings=settings)
        dictionary = self._get_dictionary(settings=settings)
        translated = []
        original = []
        for sentence in sentences:
            original_tokens, simplified_tokens = self._simplify_split_align(sentence, settings=settings)
            translated_chunk = []
            original_chunk = []
            last_token_index = len(simplified_tokens) - 1
            skip_next_token = False
            for i, word in enumerate(simplified_tokens):
                next_word = simplified_tokens[i + 1] if i < last_token_index else ""
                current_and_next_joined = self._join_chunk([word, next_word], settings=settings)
                if skip_next_token:
                    skip_next_token = False
                    continue

                if word == '' or word == ' ':
                    translated_chunk.append(word)
                    original_chunk.append(original_tokens[i])
                elif (
                    current_and_next_joined in dictionary
                    and word not in dashes
                    and self.shortname not in word_joint_unsupported_languages
                ):
                    translated_chunk.append(dictionary[current_and_next_joined])
                    original_chunk.append(
                        self._join_chunk([original_tokens[i], original_tokens[i + 1]], settings=settings)
                    )
                    skip_next_token = True
                elif word in dictionary and word not in dashes:
                    translated_chunk.append(dictionary[word])
                    original_chunk.append(original_tokens[i])
                elif word.strip('()\"\'{}[],.،') in dictionary and word not in dashes:
                    punct = word[len(word.strip('()\"\'{}[],.،')):]
                    if punct and dictionary[word.strip('()\"\'{}[],.،')]:
                        translated_chunk.append(dictionary[word.strip('()\"\'{}[],.،')] + punct)
                    else:
                        translated_chunk.append(dictionary[word.strip('()\"\'{}[],.،')])
                    original_chunk.append(original_tokens[i])
                elif self._token_with_digits_is_ok(word):
                    translated_chunk.append(word)
                    original_chunk.append(original_tokens[i])
                # Use original token because word_is_tz is case sensitive
                elif translated_chunk and word_is_tz(original_tokens[i]):
                    translated_chunk.append(word)
                    original_chunk.append(original_tokens[i])
                else:
                    if translated_chunk:
                        translated.append(translated_chunk)
                        translated_chunk = []
                        original.append(original_chunk)
                        original_chunk = []
            if translated_chunk:
                translated.append(translated_chunk)
                original.append(original_chunk)
        for i in range(len(translated)):
            if "in" in translated[i]:
                translated[i] = self._clear_future_words(translated[i])
            translated[i] = self._join_chunk(list(filter(bool, translated[i])), settings=settings)
            original[i] = self._join_chunk(list(filter(bool, original[i])), settings=settings)
        return translated, original

    def _get_abbreviations(self, settings):
        dictionary = self._get_dictionary(settings=settings)
        abbreviations = []
        if self._abbreviations is None:
            for item in dictionary:
                if item.endswith('.') and len(item) > 1:
                    abbreviations.append(item)
            self._abbreviations = abbreviations
        return self._abbreviations

    def _sentence_split(self, string, settings):
        abbreviations = self._get_abbreviations(settings=settings)
        digit_abbreviations = ['[0-9]']  # numeric date with full stop
        abbreviation_string = ''

        for abbreviation in abbreviations:
            abbreviation_string += '(?<! ' + abbreviation[:-1] + ')'  # negative lookbehind
        if self.shortname in ['fi', 'cs', 'hu', 'de', 'da']:
            for digit_abbreviation in digit_abbreviations:
                abbreviation_string += '(?<!' + digit_abbreviation + ')'  # negative lookbehind

        splitters_dict = {1: r'[\.!?;…\r\n]+(?:\s|$)*',  # most European, Tagalog, Hebrew, Georgian,
                          # Indonesian, Vietnamese
                          2: r'[\.!?;…\r\n]+(\s*[¡¿]*|$)|[¡¿]+',  # Spanish
                          3: r'[|!?;\r\n]+(?:\s|$)+',  # Hindi and Bangla
                          4: r'[。…‥\.!?？！;\r\n]+(?:\s|$)+',  # Japanese and Chinese
                          5: r'[\r\n]+',  # Thai
                          6: r'[\r\n؟!\.…]+(?:\s|$)+'}  # Arabic and Farsi
        if 'sentence_splitter_group' not in self.info:
            split_reg = abbreviation_string + splitters_dict[1]
            sentences = re.split(split_reg, string)
        else:
            split_reg = abbreviation_string + splitters_dict[self.info['sentence_splitter_group']]
            sentences = re.split(split_reg, string)

        sentences = filter(None, sentences)
        return sentences

    def _simplify_split_align(self, original, settings):
        # TODO: Switch to new split method.
        original_tokens = self._word_split(original, settings=settings)
        simplified_tokens = self._word_split(self._simplify(normalize_unicode(original), settings=settings),
                                             settings=settings)
        if len(original_tokens) == len(simplified_tokens):
            return original_tokens, simplified_tokens

        elif len(original_tokens) < len(simplified_tokens):
            add_empty = False
            for i, token in enumerate(simplified_tokens):
                if i < len(original_tokens):
                    if token == normalize_unicode(original_tokens[i].lower()):
                        add_empty = False
                    else:
                        if not add_empty:
                            add_empty = True
                            continue
                        else:
                            original_tokens.insert(i, '')
                else:
                    original_tokens.insert(i, '')
        else:
            add_empty = False
            for i, token in enumerate(original_tokens):
                if i < len(simplified_tokens):
                    if normalize_unicode(token.lower()) == simplified_tokens[i]:
                        add_empty = False
                    else:
                        if not add_empty:
                            add_empty = True
                            continue
                        else:
                            simplified_tokens.insert(i, '')
                else:
                    simplified_tokens.insert(i, '')

        while len(original_tokens) != len(simplified_tokens):
            if len(original_tokens) > len(simplified_tokens):
                original_tokens.remove('')
            else:
                simplified_tokens.remove('')
        return original_tokens, simplified_tokens

    def _get_split_dictionary(self, settings):
        if self._split_dictionary is None:
            settings.NORMALIZE = True
            dictionary = self._get_dictionary(settings=settings)
            self._split_dictionary = self._split_dict(dictionary)
        return self._split_dictionary

    def _split_dict(self, dictionary):
        newdict = {}
        for item in dictionary:
            if ' ' in item:
                items = item.split()
                for i in items:
                    newdict[i] = dictionary[item]
            else:
                newdict[item] = dictionary[item]
        return newdict

    def _word_split(self, string, settings):
        if 'no_word_spacing' in self.info:
            return self._split(string, keep_formatting=True, settings=settings)
        else:
            return string.split()

    def _split(self, date_string, keep_formatting, settings=None):
        tokens = [date_string]
        tokens = list(self._split_tokens_with_regex(tokens, r"(\d+)"))
        tokens = list(
            self._split_tokens_by_known_words(tokens, keep_formatting, settings=settings))
        return tokens

    def _split_tokens_with_regex(self, tokens, regex):
        tokens = tokens[:]
        for i, token in enumerate(tokens):
            tokens[i] = re.split(regex, token)
        return filter(bool, chain.from_iterable(tokens))

    def _split_tokens_by_known_words(self, tokens, keep_formatting, settings=None):
        dictionary = self._get_dictionary(settings)
        for i, token in enumerate(tokens):
            tokens[i] = dictionary.split(token, keep_formatting)
        return list(chain.from_iterable(tokens))

    def _join_chunk(self, chunk, settings):
        if 'no_word_spacing' in self.info:
            return self._join(chunk, separator="", settings=settings)
        else:
            return re.sub(r'\s{2,}', ' ', " ".join(chunk))

    def _token_with_digits_is_ok(self, token):
        if 'no_word_spacing' in self.info:
            if re.search(r'[\d\.:\-/]+', token) is not None:
                return True
            else:
                return False

        else:
            if re.search(r'\d+', token) is not None:
                return True
            else:
                return False

    def _simplify(self, date_string, settings=None):
        date_string = date_string.lower()
        simplifications = self._get_simplifications(settings=settings)
        for simplification in simplifications:
            pattern, replacement = list(simplification.items())[0]
            date_string = pattern.sub(replacement, date_string).lower()
        return date_string

    def _get_simplifications(self, settings=None):
        no_word_spacing = eval(self.info.get('no_word_spacing', 'False'))
        if settings.NORMALIZE:
            if self._normalized_simplifications is None:
                self._normalized_simplifications = []
                simplifications = self._generate_simplifications(normalize=True)
                for simplification in simplifications:
                    pattern, replacement = list(simplification.items())[0]
                    if not no_word_spacing:
                        pattern = r'(?<=\A|\W|_)%s(?=\Z|\W|_)' % pattern
                    pattern = re.compile(pattern, flags=re.I | re.U)
                    self._normalized_simplifications.append({pattern: replacement})
            return self._normalized_simplifications

        else:
            if self._simplifications is None:
                self._simplifications = []
                simplifications = self._generate_simplifications(normalize=False)
                for simplification in simplifications:
                    pattern, replacement = list(simplification.items())[0]
                    if not no_word_spacing:
                        pattern = r'(?<=\A|\W|_)%s(?=\Z|\W|_)' % pattern
                    pattern = re.compile(pattern, flags=re.I | re.U)
                    self._simplifications.append({pattern: replacement})
            return self._simplifications

    def _generate_simplifications(self, normalize=False):
        simplifications = []
        for simplification in self.info.get('simplifications', []):
            c_simplification = {}
            key, value = list(simplification.items())[0]
            if normalize:
                key = normalize_unicode(key)

            if isinstance(value, int):
                c_simplification[key] = str(value)
            else:
                c_simplification[key] = normalize_unicode(value) if normalize else value

            simplifications.append(c_simplification)
        return simplifications

    def _clear_future_words(self, words):
        freshness_words = {'day', 'week', 'month', 'year', 'hour', 'minute', 'second'}
        if set(words).isdisjoint(freshness_words):
            words.remove("in")
        return words

    def _join(self, tokens, separator=" ", settings=None):
        if not tokens:
            return ""

        capturing_splitters = self._get_splitters(settings)['capturing']
        joined = tokens[0]
        for i in range(1, len(tokens)):
            left, right = tokens[i - 1], tokens[i]
            if left not in capturing_splitters and right not in capturing_splitters:
                joined += separator
            joined += right

        return joined

    def _get_dictionary(self, settings=None):
        if not settings.NORMALIZE:
            if self._dictionary is None:
                self._generate_dictionary()
            self._dictionary._settings = settings
            return self._dictionary
        else:
            if self._normalized_dictionary is None:
                self._generate_normalized_dictionary()
            self._normalized_dictionary._settings = settings
            return self._normalized_dictionary

    def _get_wordchars(self, settings=None):
        if self._wordchars is None:
            self._set_wordchars(settings)
        return self._wordchars

    def _get_splitters(self, settings=None):
        if self._splitters is None:
            self._set_splitters(settings)
        return self._splitters

    def _set_splitters(self, settings=None):
        splitters = {
            # The ones that split string only if they are not surrounded by letters from both sides:
            'wordchars': set(),
            # The ones that are not filtered out from tokens after split:
            'capturing': set(),
        }
        splitters['capturing'] |= set(ALWAYS_KEEP_TOKENS)

        wordchars = self._get_wordchars(settings)
        skip = set(self.info.get('skip', [])) | splitters['capturing']
        for token in skip:
            if not re.match(r'^\W+$', token, re.UNICODE):
                continue
            if token in wordchars:
                splitters['wordchars'].add(token)

        self._splitters = splitters

    def _set_wordchars(self, settings=None):
        wordchars = set()
        for word in self._get_dictionary(settings):
            if re.match(r'^[\W\d_]+$', word, re.UNICODE):
                continue
            for char in word:
                wordchars.add(char.lower())

        self._wordchars = wordchars - {" "} | {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"}

    def get_wordchars_for_detection(self, settings):
        if self._wordchars_for_detection is None:
            wordchars = set()
            for word in self._get_dictionary(settings):
                if re.match(r'^[\W\d_]+$', word, re.UNICODE):
                    continue
                for char in word:
                    wordchars.add(char.lower())
            self._wordchars_for_detection = wordchars - {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
                                                         ":", "(", ")", "'", "q", "a", "m", "p", " "}
        return self._wordchars_for_detection

    def _generate_dictionary(self, settings=None):
        self._dictionary = Dictionary(self.info, settings=settings)

    def _generate_normalized_dictionary(self, settings=None):
        self._normalized_dictionary = NormalizedDictionary(self.info, settings=settings)

    def to_parserinfo(self, base_cls=parser.parserinfo):
        attributes = {
            'JUMP': self.info.get('skip', []),
            'PERTAIN': self.info.get('pertain', []),
            'WEEKDAYS': [self.info['monday'],
                         self.info['tuesday'],
                         self.info['wednesday'],
                         self.info['thursday'],
                         self.info['friday'],
                         self.info['saturday'],
                         self.info['sunday']],
            'MONTHS': [self.info['january'],
                       self.info['february'],
                       self.info['march'],
                       self.info['april'],
                       self.info['may'],
                       self.info['june'],
                       self.info['july'],
                       self.info['august'],
                       self.info['september'],
                       self.info['october'],
                       self.info['november'],
                       self.info['december']],
            'HMS': [self.info['hour'],
                    self.info['minute'],
                    self.info['second']],
        }
        name = '{language}ParserInfo'.format(language=self.info['name'])
        return type(name, bases=[base_cls], dict=attributes)
